from __future__ import print_function
from IMP import ArgumentParser
import os

__doc__ = "Perform analysis to determine sampling convergence."

############################################################
# Scripts written by Shruthi Viswanath and Ilan E. Chemmama#
#              in Andrej Sali Lab at UCSF.                 #
#   Based on Viswanath, Chemmama et al. Biophys. J. (2017) #
#                                                          #
############################################################


def parse_args():
    parser = ArgumentParser(
        description="First stages of analysis for assessing sampling "
                    "convergence")
    parser.add_argument(
        '--sysname', '-n', dest="sysname",
        help='name of the system', default="")
    parser.add_argument(
        '--path', '-p', dest="path",
        help='path to the good-scoring models', default="./")
    parser.add_argument(
        '--extension', '-e', dest="extension",
        help='extension of the file', choices=['rmf', 'pdb'], default="rmf")
    parser.add_argument(
        '--mode', '-m', dest="mode", help='pyRMSD calculator',
        choices=['cuda', 'cpu_omp', 'cpu_serial'], default="cuda")
    parser.add_argument(
        '--cores', '-c', dest="cores", type=int,
        help='number of cores for RMSD matrix calculations; '
             'only for cpu_omp', default=1)
    parser.add_argument(
        '--subunit', '-su', dest="subunit",
        help='calculate RMSD/sampling and cluster precision/densities '
             'etc over this subunit only', default=None)
    parser.add_argument(
        '--align', '-a', dest="align",
        help='boolean flag to allow superposition of models',
        default=False, action='store_true')
    parser.add_argument(
        '--ambiguity', '-amb', dest="symmetry_groups",
        help='file containing symmetry groups', default=None)
    parser.add_argument(
        '--scoreA', '-sa', dest="scoreA",
        help='name of the file having the good-scoring scores for sample A',
        default="scoresA.txt")
    parser.add_argument(
        '--scoreB', '-sb', dest="scoreB",
        help='name of the file having the good-scoring scores for sample B',
        default="scoresB.txt")
    parser.add_argument(
        '--rmfA', '-ra', dest="rmf_A",
        help='RMF file with conformations from Sample A', default=None)
    parser.add_argument(
        '--rmfB', '-rb', dest="rmf_B",
        help='RMF file with conformations from Sample B', default=None)
    parser.add_argument(
        '--gridsize', '-g', dest="gridsize", type=float,
        help='grid size for calculating sampling precision', default=10.0)
    parser.add_argument(
        '--skip', '-s', dest="skip_sampling_precision",
        help="This option will bypass the calculation of sampling "
             "precision. This option needs to be used with the clustering "
             "threshold option. Otherwise by default, sampling precision "
             "is calculated and the clustering threshold is the "
             "calculated sampling precision.", default=False,
        action='store_true')
    parser.add_argument(
        '--cluster_threshold', '-ct', dest="cluster_threshold", type=float,
        help='final clustering threshold to visualize clusters. Assumes '
             'that the user has previously calculated sampling precision '
             'and wants clusters defined at a threshold higher than the '
             'sampling precision for ease of analysis (lesser number of '
             'clusters).', default=30.0)
    parser.add_argument(
        '--voxel', '-v', dest="voxel", type=float,
        help='voxel size for the localization densities', default=5.0)
    parser.add_argument(
        '--density_threshold', '-dt', type=float,
        dest="density_threshold",
        help='threshold for localization densities', default=20.0)
    parser.add_argument(
        '--density', '-d', dest="density",
        help='file containing dictionary of density custom ranges',
        default=None)
    parser.add_argument(
        '--gnuplot', '-gp', dest="gnuplot",
        help="plotting automatically with gnuplot", default=False,
        action='store_true')
    parser.add_argument(
        '--selection', '-sn', dest="selection",
        help='file containing dictionary'
        'of selected subunits and residues'
        'for RMSD and clustering calculation'
        "each entry in the dictionary takes the form"
        "'selection name': [(residue_start, residue_end, protein name)",
        default=None)
    return parser.parse_args()


def make_cluster_centroid(infname, frame, outfname, cluster_index,
                          cluster_size, precision, density, path):

    import RMF
    # If we have new enough IMP/RMF, do our own RMF slicing with provenance
    if hasattr(RMF.NodeHandle, 'replace_child'):
        print(infname, outfname)
        inr = RMF.open_rmf_file_read_only(infname)
        outr = RMF.create_rmf_file(outfname)
        cpf = RMF.ClusterProvenanceFactory(outr)
        RMF.clone_file_info(inr, outr)
        RMF.clone_hierarchy(inr, outr)
        RMF.clone_static_frame(inr, outr)
        inr.set_current_frame(RMF.FrameID(frame))
        outr.add_frame("f0")
        RMF.clone_loaded_frame(inr, outr)
        rn = outr.get_root_node()
        children = rn.get_children()
        if len(children) == 0:
            return
        rn = children[0]  # Should be the top-level IMP node
        prov = [c for c in rn.get_children() if c.get_type() == RMF.PROVENANCE]
        if not prov:
            return
        prov = prov[0]
        # Add cluster-provenance info
        newp = rn.replace_child(
            prov, "cluster.%d" % cluster_index, RMF.PROVENANCE)
        cp = cpf.get(newp)
        cp.set_members(cluster_size)
        cp.set_precision(precision)
        cp.set_density(os.path.abspath(density))
    else:
        # Otherwise, fall back to RMF's command line tool
        import subprocess
        print(infname, frame, outfname)
        subprocess.call(['rmf_slice', path + infname, '-f', str(frame),
                         outfname])


def main():
    args = parse_args()

    import os
    import shutil
    import numpy

    import scipy as sp

    import IMP.sampcon
    from IMP.sampcon import scores_convergence, clustering_rmsd
    from IMP.sampcon import rmsd_calculation, precision_rmsd

    import IMP

    idfile_A = "Identities_A.txt"
    idfile_B = "Identities_B.txt"

    # Step 0: Compute Score convergence
    score_A = []
    score_B = []

    with open(os.path.join(args.path, args.scoreA), 'r') as f:
        for line in f:
            score_A.append(float(line.strip("\n")))

    with open(os.path.join(args.path, args.scoreB), 'r') as f:
        for line in f:
            score_B.append(float(line.strip("\n")))

    scores = score_A + score_B

    # Get the convergence of the best score
    scores_convergence.get_top_scorings_statistics(scores, 0, args.sysname)

    # Check if the two score distributions are similar
    scores_convergence.get_scores_distributions_KS_Stats(
            score_A, score_B, 100, args.sysname)

    # Step 1: Compute RMSD matrix
    if args.extension == "pdb":
        ps_names = []  # bead names are not stored in PDB files
        symm_groups = None
        conforms, masses, radii, models_name = \
            rmsd_calculation.get_pdbs_coordinates(
                args.path, idfile_A, idfile_B)
    else:
        args.extension = "rmf3"
        if args.selection is not None:
            rmsd_custom_ranges = \
                precision_rmsd.parse_custom_ranges(args.selection)
        else:
            rmsd_custom_ranges = None
        # If we have a single RMF file, read conformations from that
        if args.rmf_A is not None:
            (ps_names, masses, radii, conforms, symm_groups, models_name,
                n_models) = rmsd_calculation.get_rmfs_coordinates_one_rmf(
                     args.path, args.rmf_A, args.rmf_B, args.subunit,
                     args.symmetry_groups,
                     rmsd_custom_ranges)

        # If not, default to the Identities.txt file
        else:
            symm_groups = None
            (ps_names, masses, radii, conforms,
             models_name) = rmsd_calculation.get_rmfs_coordinates(
                     args.path, idfile_A, idfile_B, args.subunit,
                     selection=rmsd_custom_ranges)

    print("Size of conformation matrix", conforms.shape)

    if not args.skip_sampling_precision:
        # get_rmsds_matrix modifies conforms, so save it to a file and restore
        # afterwards (so that we retain the original IMP orientation)
        numpy.save("conforms", conforms)
        inner_data = rmsd_calculation.get_rmsds_matrix(
                conforms, args.mode, args.align, args.cores, symm_groups)
        print("Size of RMSD matrix (flattened):", inner_data.shape)
        del conforms
        conforms = numpy.load("conforms.npy")
        os.unlink('conforms.npy')

    from pyRMSD.matrixHandler import MatrixHandler
    mHandler = MatrixHandler()
    mHandler.loadMatrix("Distances_Matrix.data")

    rmsd_matrix = mHandler.getMatrix()
    distmat = rmsd_matrix.get_data()

    distmat_full = sp.spatial.distance.squareform(distmat)
    print("Size of RMSD matrix (unpacked, N x N):", distmat_full.shape)

    # Get model lists
    if args.rmf_A is not None:
        sampleA_all_models = list(range(n_models[0]))
        sampleB_all_models = list(range(n_models[0],
                                        n_models[1] + n_models[0]))
        total_num_models = n_models[1] + n_models[0]
    else:
        (sampleA_all_models,
         sampleB_all_models) = clustering_rmsd.get_sample_identity(
                idfile_A, idfile_B)
        total_num_models = len(sampleA_all_models) + len(sampleB_all_models)
    all_models = list(sampleA_all_models) + list(sampleB_all_models)
    print("Size of Sample A:", len(sampleA_all_models),
          " ; Size of Sample B: ", len(sampleB_all_models),
          "; Total", total_num_models)

    if not args.skip_sampling_precision:

        print("Calculating sampling precision")

        # Step 2: Cluster at intervals of grid size to get the
        # sampling precision
        gridSize = args.gridsize

        # Get cutoffs for clustering
        cutoffs_list = clustering_rmsd.get_cutoffs_list(distmat, gridSize)
        print("Clustering at thresholds:", cutoffs_list)

        # Do clustering at each cutoff
        pvals, cvs, percents = clustering_rmsd.get_clusters(
                cutoffs_list, distmat_full, all_models, total_num_models,
                sampleA_all_models, sampleB_all_models, args.sysname)

        # Now apply the rule for selecting the right precision based
        # on population of contingency table, pvalue and cramersv
        (sampling_precision, pval_converged, cramersv_converged,
         percent_converged) = clustering_rmsd.get_sampling_precision(
                 cutoffs_list, pvals, cvs, percents)

        # Output test statistics
        with open("%s.Sampling_Precision_Stats.txt"
                  % args.sysname, 'w+') as fpv:
            print("The sampling precision is defined as the largest allowed "
                  "RMSD between the cluster centroid and a ", args.sysname,
                  "model within any cluster in the finest clustering for "
                  "which each sample contributes models proportionally to "
                  "its size (considering both significance and magnitude of "
                  "the difference) and for which a sufficient proportion of "
                  "all models occur in sufficiently large clusters. The "
                  "sampling precision for our ", args.sysname,
                  " modeling is %.3f" % (sampling_precision), " A.", file=fpv)

            print("Sampling precision, P-value, Cramer's V and percentage "
                  "of clustered models below:", file=fpv)
            print("%.3f\t%.3f\t%.3f\t%.3f"
                  % (sampling_precision, pval_converged, cramersv_converged,
                     percent_converged), file=fpv)
            print("", file=fpv)

        final_clustering_threshold = sampling_precision

    else:
        final_clustering_threshold = args.cluster_threshold

    # Perform final clustering at the required precision
    print("Clustering at threshold %.3f" % final_clustering_threshold)
    (cluster_centers, cluster_members) = clustering_rmsd.precision_cluster(
            distmat_full, total_num_models, final_clustering_threshold)

    (ctable, retained_clusters) = clustering_rmsd.get_contingency_table(
            len(cluster_centers), cluster_members, all_models,
            sampleA_all_models, sampleB_all_models)
    print("Contingency table:", ctable)
    # Output the number of models in each cluster and each sample
    with open("%s.Cluster_Population.txt" % args.sysname, 'w+') as fcp:
        for rows in range(len(ctable)):
            print(rows, ctable[rows][0], ctable[rows][1], file=fcp)

    # Obtain the subunits for which we need to calculate densities
    density_custom_ranges = precision_rmsd.parse_custom_ranges(args.density)

    # Output cluster precisions
    fpc = open("%s.Cluster_Precision.txt" % args.sysname, 'w+')

    # For each cluster, output the models in the cluster
    # Also output the densities for the cluster models
    for i in range(len(retained_clusters)):
        clus = retained_clusters[i]

        # The cluster centroid is the first conformation.
        # We use this as to align and compute RMSD/precision
        conform_0 = conforms[all_models[cluster_members[clus][0]]]

        # create a directory for the cluster
        if not os.path.exists("./cluster.%s" % i):
            os.mkdir("./cluster.%s" % i)
            os.mkdir("./cluster.%s/Sample_A/" % i)
            os.mkdir("./cluster.%s/Sample_B/" % i)
        else:
            shutil.rmtree("./cluster.%s" % i)
            os.mkdir("./cluster.%s" % i)
            os.mkdir("./cluster.%s/Sample_A/" % i)
            os.mkdir("./cluster.%s/Sample_B/" % i)

        # Create densities for all subunits for both sample A and sample B
        # as well as separately.
        gmd1 = precision_rmsd.GetModelDensity(
                custom_ranges=density_custom_ranges,
                resolution=args.density_threshold, voxel=args.voxel,
                bead_names=ps_names)
        gmd2 = precision_rmsd.GetModelDensity(
                custom_ranges=density_custom_ranges,
                resolution=args.density_threshold, voxel=args.voxel,
                bead_names=ps_names)
        gmdt = precision_rmsd.GetModelDensity(
                custom_ranges=density_custom_ranges,
                resolution=args.density_threshold, voxel=args.voxel,
                bead_names=ps_names)

        # Also output the identities of cluster members
        both_file = open('cluster.'+str(i)+'.all.txt', 'w')
        sampleA_file = open('cluster.'+str(i)+'.sample_A.txt', 'w')
        sampleB_file = open('cluster.'+str(i)+'.sample_B.txt', 'w')

        # Create a model with just the cluster_member particles
        model = IMP.Model()
        ps = []  # particle list to be updated by each RMF frame
        for pi in range(len(conform_0)):
            p = IMP.Particle(model, "%s" % str(pi))
            IMP.core.XYZ.setup_particle(p, (0, 0, 0))
            IMP.core.XYZR.setup_particle(p, float(radii[pi]))
            IMP.atom.Mass.setup_particle(p, float(masses[pi]))
            ps.append(p)

        # Obtain cluster precision by obtaining average RMSD of each model
        # to the cluster center
        cluster_precision = 0.0

        # transformation from internal pyRMSD orientation
        trans = None
        # for each model in the cluster
        for mem in cluster_members[clus]:

            model_index = all_models[mem]

            # get superposition of each model to cluster center and the
            # RMSD between the two
            if args.symmetry_groups:
                rmsd, superposed_ps, trans = \
                    precision_rmsd.get_particles_from_superposed_amb(
                        conforms[model_index], conform_0, args.align, ps,
                        trans, symm_groups)
            else:
                rmsd, superposed_ps, trans = \
                    precision_rmsd.get_particles_from_superposed(
                        conforms[model_index], conform_0, args.align,
                        ps, trans)

            model.update()  # why not?

            cluster_precision += rmsd

            # Add the superposed particles to the respective density maps
            gmdt.add_subunits_density(superposed_ps)  # total density map
            print(model_index, file=both_file)

            if model_index in sampleA_all_models:
                # density map for sample A
                gmd1.add_subunits_density(superposed_ps)
                print(model_index, file=sampleA_file)
            else:
                # density map for sample B
                gmd2.add_subunits_density(superposed_ps)
                print(model_index, file=sampleB_file)

        cluster_precision /= float(len(cluster_members[clus]) - 1.0)

        print("Cluster precision (average distance to cluster centroid) "
              "of cluster ", str(i), " is %.3f" % cluster_precision, "A",
              file=fpc)

        both_file.close()
        sampleA_file.close()
        sampleB_file.close()

        # Output density files for the cluster
        density = gmdt.write_mrc(path="./cluster.%s" % i, file_prefix="LPD")
        gmd1.write_mrc(path="./cluster.%s/Sample_A/" % i, file_prefix="LPD")
        gmd2.write_mrc(path="./cluster.%s/Sample_B/" % i, file_prefix="LPD")

        # Add the cluster center model RMF to the cluster directory
        cluster_center_index = cluster_members[clus][0]
        if args.rmf_A is not None:
            cluster_center_model_id = cluster_center_index
            if cluster_center_index < n_models[0]:
                make_cluster_centroid(
                    os.path.join(args.path, args.rmf_A),
                    cluster_center_index,
                    os.path.join("cluster.%d" % i,
                                 "cluster_center_model.rmf3"),
                    i, len(cluster_members[clus]),
                    cluster_precision, density, args.path)
            else:
                make_cluster_centroid(
                    os.path.join(args.path, args.rmf_B),
                    cluster_center_index - n_models[0],
                    os.path.join("cluster.%d" % i,
                                 "cluster_center_model.rmf3"),
                    i, len(cluster_members[clus]),
                    cluster_precision, density, args.path)
        else:
            # index to Identities file.
            cluster_center_model_id = all_models[cluster_center_index]
            outfname = os.path.join("cluster.%d" % i,
                                    "cluster_center_model." + args.extension)
            if 'rmf' in args.extension:
                make_cluster_centroid(
                        models_name[cluster_center_model_id], 0, outfname,
                        i, len(cluster_members[clus]),
                        cluster_precision, density, args.path)
            else:
                shutil.copy(models_name[cluster_center_model_id], outfname)

    fpc.close()

    # generate plots for the score and structure tests
    if args.gnuplot:
        import subprocess
        import glob

        gnuplotdir = IMP.sampcon.get_data_path("gnuplot_scripts")
        for filename in sorted(glob.glob(os.path.join(gnuplotdir, "*.plt"))):
            cmd = ['gnuplot', '-e', 'sysname="%s"' % args.sysname, filename]
            print(" ".join(cmd))
            subprocess.check_call(cmd)


if __name__ == '__main__':
    main()



"""@namespace IMP.mmcif.util
   @brief Utility functions for IMP.mmcif.
"""

import ihm.location
import ihm.dumper
import IMP.mmcif.data
import IMP.mmcif.restraint
import IMP.rmf
import IMP.atom
import RMF
import ihm.format
import ihm.representation
import string
import weakref
import operator


class _ChainIDs(object):
    """Map indices to multi-character chain IDs.
       We label the first 26 chains A-Z, then we move to two-letter
       chain IDs: AA through AZ, then BA through BZ, through to ZZ.
       This continues with longer chain IDs."""
    def __getitem__(self, ind):
        chars = string.ascii_uppercase
        lc = len(chars)
        ids = []
        while ind >= lc:
            ids.append(chars[ind % lc])
            ind = ind // lc - 1
        ids.append(chars[ind])
        return "".join(reversed(ids))


class RMFFrame(object):
    """An individual state conformation read from a PDB file"""
    def __init__(self, filename, frame, name):
        self.filename, self.frame = filename, frame
        self.name = name

    def create(self, model):
        rmf = RMF.open_rmf_file_read_only(self.filename)
        # todo: support frame!=0
        hiers = IMP.rmf.create_hierarchies(rmf, model)
        restraints = IMP.rmf.create_restraints(rmf, model)
        return hiers, restraints

    def link(self, hiers, restraints):
        rmf = RMF.open_rmf_file_read_only(self.filename)
        IMP.rmf.link_hierarchies(rmf, hiers)
        IMP.rmf.link_restraints(rmf, restraints)
        IMP.rmf.load_frame(rmf, RMF.FrameID(self.frame))


class _ModelFrame(object):
    """An individual state conformation read from an IMP.Model"""
    def __init__(self, hiers, restraints, name):
        self.hiers, self.restraints = hiers, restraints
        self.name = name

    def create(self, model):
        return self.hiers, self.restraints

    def link(self, hiers, restraints):
        if len(hiers) != len(self.hiers) \
           or len(restraints) != len(self.restraints):
            raise ValueError("Frames do not match")
        hiers[:] = self.hiers
        # todo: this won't work currently because the Restraint objects
        # will change
        restraints[:] = self.restraints


class _NonModeledChain(object):
    """Represent a chain that was experimentally characterized but not modeled.
       Such a chain resembles an IMP.atom.Chain, but has no associated
       structure, and belongs to no state."""
    def __init__(self, name, sequence, chain_type):
        self.name = name
        self.sequence = sequence
        self.chain_type = chain_type

    def get_sequence(self):
        return self.sequence


class System(object):
    def __init__(self):
        self.system = ihm.System()
        self._states = []
        self._ensembles = []
        self._frames = []

        self.entities = IMP.mmcif.data._EntityMapper(self.system)
        self.components = IMP.mmcif.data._ComponentMapper(self.system)
        self._software = IMP.mmcif.data._AllSoftware(self.system)
        self._external_files = IMP.mmcif.data._ExternalFiles(self.system)
        self.datasets = IMP.mmcif.data._Datasets(self.system)
        # All modeling protocols
        self.protocols = IMP.mmcif.data._Protocols(self.system)
        self.representation = ihm.representation.Representation()
        self.system.orphan_representations.append(self.representation)

    def _update_location(self, fileloc):
        """Update FileLocation to point to a parent repository, if any"""
        ihm.location.Repository._update_in_repos(fileloc,
                                                 self._external_files._repos)

    def add_repository(self, doi, root=None, url=None, top_directory=None):
        """Add a repository containing one or more modeling files."""
        self._external_files.add_repo(ihm.location.Repository(
                                            doi, root, url, top_directory))

    def _add_state(self, state):
        if not self.system.state_groups:
            self.system.state_groups.append(ihm.model.StateGroup())
        self.system.state_groups[-1].append(state)
        self._states.append(state)

    def _add_ensemble(self, ensemble):
        self._ensembles.append(ensemble)
        self.system.ensembles.append(ensemble)

    def _add_frame(self, frame):
        self._frames.append(frame)
        frame.id = len(self._frames)

    def _add_hierarchy(self, h, state):
        chains = [IMP.atom.Chain(c)
                  for c in IMP.atom.get_by_type(h, IMP.atom.CHAIN_TYPE)]
        if len(chains) == 0:
            raise ValueError("No chains found in %s" % h)
        # todo: handle same chain in multiple states
        for c in chains:
            component = self._add_chain(c)
            state._all_modeled_components.append(component)
            if hasattr(component, 'asym_unit'):
                state.modeled_assembly.append(component.asym_unit)
            else:
                state.modeled_assembly.append(component.entity)
            state.repsegments[component] = \
                list(self._get_repsegments(
                    c, component, self._get_all_starting_models(component)))
            # Number of states that have representation for this component
            num_state_reps = len([s for s in self._states
                                  if component in s.repsegments])
            # Assume representation for a given component is the same in all
            # states, so we only need one copy of it in the mmCIF file
            if num_state_reps == 1:
                self.representation.extend(state.repsegments[component])
        self.protocols._add_hierarchy(h, state.modeled_assembly)
        self._external_files.add_hierarchy(h)
        self._software.add_hierarchy(h)

    def _get_all_starting_models(self, comp):
        """Get all starting models (in all states) for the given component"""
        for state in self._states:
            for seg in state.repsegments.get(comp, []):
                if seg.starting_model:
                    yield seg.starting_model

    def _get_repsegments(self, chain, component, existing_starting_models):
        """Yield groups of particles under chain with same representation"""
        smf = IMP.mmcif.data._StartingModelFinder(component,
                                                  existing_starting_models)
        segfactory = IMP.mmcif.data._RepSegmentFactory(component)

        for sp in self._get_structure_particles(chain):
            starting_model = smf.find(sp, self)
            seg = segfactory.add(sp, starting_model)
            if seg:
                yield seg
        last = segfactory.get_last()
        if last:
            yield last

    def _get_structure_particles(self, chain):
        """Yield all particles under chain with coordinates.
           They are sorted by residue index."""
        # todo: handle Representation decorators for non-PMI-1 models
        ps = IMP.atom.get_leaves(chain)
        resind_dict = {}
        for p in ps:
            if IMP.atom.Residue.get_is_setup(p):
                residue = IMP.atom.Residue(p)
                resind = residue.get_index()
                if resind in resind_dict:
                    continue
                resind_dict[resind] = residue
            elif IMP.atom.Fragment.get_is_setup(p):
                fragment = IMP.atom.Fragment(p)
                # todo: handle non-contiguous fragments
                resinds = fragment.get_residue_indexes()
                resind = resinds[len(resinds) // 2]
                if resind in resind_dict:
                    continue
                resind_dict[resind] = fragment
        # Return values sorted by key (residue index)
        for item in sorted(resind_dict.items(), key=operator.itemgetter(0)):
            yield item[1]

    def add_non_modeled_chain(self, name, sequence,
                              chain_type=IMP.atom.UnknownChainType):
        """Add a chain that wasn't modeled by IMP."""
        c = _NonModeledChain(name, sequence, chain_type)
        self._add_chain(c)

    def _add_chain(self, c):
        entity = self.entities.add(c)
        component = self.components.add(c, entity)
        return component

    def write(self, fname):
        with open(fname, 'w') as fh:
            ihm.dumper.write(fh, [self.system])


class State(ihm.model.State):
    """Represent a single IMP state."""
    def __init__(self, system):
        super(State, self).__init__()
        self.system = weakref.proxy(system)
        system._add_state(self)
        self.model = IMP.Model()
        self.hiers = None
        self._wrapped_restraints = []
        # The assembly of all components modeled by IMP in this state.
        # This may be smaller than the complete assembly.
        self.modeled_assembly = ihm.Assembly(
            name="Modeled assembly",
            description="All components modeled by IMP")
        system.system.orphan_assemblies.append(self.modeled_assembly)
        # A list of ihm.representation.Segment objects for each Component
        self.repsegments = {}
        self._frames = []

        self._all_modeled_components = []

    def _add_frame(self, f, model):
        self._frames.append(f)
        self.system._add_frame(f)
        if self._load_frame(f):
            for h in self.hiers:
                self._add_hierarchy(h)
            self._add_restraints(self.restraints, model)
        else:
            self._update_restraints(model)

    def _load_frame(self, f):
        """Load hierarchies and restraints from a frame.
           Return True if this results in making new hierarchies."""
        if self.hiers is None:
            self.hiers, self.restraints = f.create(self.model)
            self._remove_duplicate_chain_ids(True)
            return True
        else:
            f.link(self.hiers, self.restraints)
            self._remove_duplicate_chain_ids(False)
            return False

    def _remove_duplicate_chain_ids(self, new_hiers):
        chains = []
        for h in self.hiers:
            chains.extend(
                IMP.atom.Chain(c)
                for c in IMP.atom.get_by_type(h, IMP.atom.CHAIN_TYPE))
        if new_hiers:
            self._assigned_chain_ids = []
            chain_ids = [c.get_id() for c in chains]
            if len(set(chain_ids)) < len(chain_ids):
                print("Duplicate chain IDs detected - reassigning "
                      "alphabetically")
                for chain, cid in zip(chains, _ChainIDs()):
                    self._assigned_chain_ids.append(cid)
                    chain.set_id(cid)
        else:
            for chain, cid in zip(chains, self._assigned_chain_ids):
                chain.set_id(cid)

    def _add_hierarchy(self, h):
        self.system._add_hierarchy(h, self)

    def _add_restraints(self, rs, model):
        m = IMP.mmcif.restraint._RestraintMapper(self.system)
        for r in rs:
            rw = m.handle(r, model, self.modeled_assembly)
            if rw:
                self._wrapped_restraints.append(rw)
                self.system.system.restraints.append(rw)

    def _update_restraints(self, model):
        for rw in self._wrapped_restraints:
            rw.add_model_fit(model)


class Ensemble(ihm.model.Ensemble):
    """Represent a set of similar models in a state."""
    def __init__(self, state, name):
        self.state = weakref.proxy(state)
        state.system._add_ensemble(self)
        self._frames = []
        mg = ihm.model.ModelGroup(name=name)
        state.append(mg)
        super(Ensemble, self).__init__(model_group=mg, num_models=0, name=name)

    def add_frame(self, frame):
        """Add a frame from a custom source"""
        self._frames.append(frame)
        self.num_models += 1
        model = IMP.mmcif.data._Model(frame, self.state)
        self.model_group.append(model)
        self.state._add_frame(frame, model)

    def add_rmf(self, fname, name, frame=0):
        """Add a frame from an RMF file"""
        self.add_frame(RMFFrame(fname, frame, name))

    def add_model(self, hiers, restraints, name):
        """Add hierarchies and restraints from an IMP.Model"""
        self.add_frame(_ModelFrame(hiers, restraints, name))



## \example atom/rigid_brownian_dynamics.py
# This example shows how to run brownian dynamics with rigid bodies.

import IMP.atom
import IMP.core
import IMP.algebra
import IMP.display
import sys

IMP.setup_from_argv(sys.argv, "rigid brownian dynamics")


def create_rigid_body(m, name):
    prb = IMP.core.RigidBody.setup_particle(
        IMP.Particle(m), IMP.algebra.ReferenceFrame3D())
    prb.set_coordinates_are_optimized(True)
    prb.set_name(name + " rb")
    ph = IMP.atom.Molecule.setup_particle(IMP.Particle(m))
    ph.set_name(name)
    for i in range(0, 2):
        for j in range(0, 2):
            for k in range(0, 2):
                d = IMP.core.XYZR.setup_particle(IMP.Particle(m),
                                                 IMP.algebra.Sphere3D(
                                                     IMP.algebra.Vector3D(
                                                         i, j, k) * 10.0,
                                                     10))
                ph.add_child(IMP.atom.Fragment.setup_particle(d))
                IMP.atom.Mass.setup_particle(d, 1000)
                prb.add_member(d)
                d.set_name(name + str(i) + str(j) + str(k))
    d = IMP.atom.RigidBodyDiffusion.setup_particle(prb)
    d.set_rotational_diffusion_coefficient(
        d.get_rotational_diffusion_coefficient() * 100)
    return prb, ph


def display(i, w, hs):
    w.set_frame(i)
    for h in hs:
        g = IMP.atom.HierarchyGeometry(h)
        w.add_geometry(g)

m = IMP.Model()
IMP.set_log_level(IMP.SILENT)

rb0, h0 = create_rigid_body(m, "first")
rb1, h1 = create_rigid_body(m, "second")
rb1.set_reference_frame(IMP.algebra.ReferenceFrame3D(
    IMP.algebra.Transformation3D(IMP.algebra.Vector3D(30, 0, 0))))

ev = IMP.core.ExcludedVolumeRestraint(
    IMP.container.ListSingletonContainer(
        m, rb0.get_rigid_members() + rb1.get_rigid_members()), 1, 3)

#h= IMP.core.Harmonic(0,1)
#s= IMP.core.DistanceToSingletonScore(h, IMP.algebra.Vector3D(0,0,0))
#r= IMP.core.SingletonRestraint(s, rb0.get_member(0))
# m.add_restraint(r)
cr = IMP.atom.create_distance_restraint(
    IMP.atom.Selection(h0), IMP.atom.Selection(h1), 0, 1)

sf = IMP.core.RestraintsScoringFunction([ev, cr])

bd = IMP.atom.BrownianDynamics(m)
bd.set_scoring_function(sf)
bd.set_time_step(10000)

nm = IMP.create_temporary_file_name("rigid_bd", ".pym")
nm = "rigid.pym"
w = IMP.display.PymolWriter(nm)
for i in range(0, 100):
    display(i, w, [h0, h1])
    bd.optimize(10)



## \example core/custom_hierarchy.py
# This example shows how to create and use a custom hierarchy of particles.

import IMP
import IMP.core
import sys

IMP.setup_from_argv(sys.argv, "custom hierarchy")

def custom_hierarchy(parent_particle, children_particles):
    tr = IMP.core.HierarchyTraits("my hierarchy")
    pd = IMP.core.Hierarchy.setup_particle(parent_particle, tr)
    for p in children_particles:
        cd = IMP.core.Hierarchy.setup_particle(p, tr)
        pd.add_child(cd)
    pd.show()

m = IMP.Model()
p1 = IMP.Particle(m)
p2 = IMP.Particle(m)
p3 = IMP.Particle(m)
custom_hierarchy(p1, (p2, p3))



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

# This wrapper is part of IMP,
# Copyright 2007-2021 IMP Inventors. All rights reserved.

from __future__ import print_function, division, absolute_import
from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

import _IMP_saxs

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class IMP_SAXS_SwigPyIterator(object):
    r"""Proxy of C++ swig::IMP_SAXS_SwigPyIterator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _IMP_saxs.delete_IMP_SAXS_SwigPyIterator

    def value(self):
        r"""value(IMP_SAXS_SwigPyIterator self) -> PyObject *"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_value(self)

    def incr(self, n=1):
        r"""incr(IMP_SAXS_SwigPyIterator self, size_t n=1) -> IMP_SAXS_SwigPyIterator"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        r"""decr(IMP_SAXS_SwigPyIterator self, size_t n=1) -> IMP_SAXS_SwigPyIterator"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_decr(self, n)

    def distance(self, x):
        r"""distance(IMP_SAXS_SwigPyIterator self, IMP_SAXS_SwigPyIterator x) -> ptrdiff_t"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_distance(self, x)

    def equal(self, x):
        r"""equal(IMP_SAXS_SwigPyIterator self, IMP_SAXS_SwigPyIterator x) -> bool"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_equal(self, x)

    def copy(self):
        r"""copy(IMP_SAXS_SwigPyIterator self) -> IMP_SAXS_SwigPyIterator"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_copy(self)

    def next(self):
        r"""next(IMP_SAXS_SwigPyIterator self) -> PyObject *"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_next(self)

    def __next__(self):
        r"""__next__(IMP_SAXS_SwigPyIterator self) -> PyObject *"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator___next__(self)

    def previous(self):
        r"""previous(IMP_SAXS_SwigPyIterator self) -> PyObject *"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_previous(self)

    def advance(self, n):
        r"""advance(IMP_SAXS_SwigPyIterator self, ptrdiff_t n) -> IMP_SAXS_SwigPyIterator"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        r"""__eq__(IMP_SAXS_SwigPyIterator self, IMP_SAXS_SwigPyIterator x) -> bool"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        r"""__ne__(IMP_SAXS_SwigPyIterator self, IMP_SAXS_SwigPyIterator x) -> bool"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        r"""__iadd__(IMP_SAXS_SwigPyIterator self, ptrdiff_t n) -> IMP_SAXS_SwigPyIterator"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        r"""__isub__(IMP_SAXS_SwigPyIterator self, ptrdiff_t n) -> IMP_SAXS_SwigPyIterator"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        r"""__add__(IMP_SAXS_SwigPyIterator self, ptrdiff_t n) -> IMP_SAXS_SwigPyIterator"""
        return _IMP_saxs.IMP_SAXS_SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        r"""
        __sub__(IMP_SAXS_SwigPyIterator self, ptrdiff_t n) -> IMP_SAXS_SwigPyIterator
        __sub__(IMP_SAXS_SwigPyIterator self, IMP_SAXS_SwigPyIterator x) -> ptrdiff_t
        """
        return _IMP_saxs.IMP_SAXS_SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register IMP_SAXS_SwigPyIterator in _IMP_saxs:
_IMP_saxs.IMP_SAXS_SwigPyIterator_swigregister(IMP_SAXS_SwigPyIterator)


_value_types=[]
_object_types=[]
_raii_types=[]
_plural_types=[]

IMP_DEBUG = _IMP_saxs.IMP_DEBUG

IMP_RELEASE = _IMP_saxs.IMP_RELEASE

IMP_SILENT = _IMP_saxs.IMP_SILENT

IMP_PROGRESS = _IMP_saxs.IMP_PROGRESS

IMP_TERSE = _IMP_saxs.IMP_TERSE

IMP_VERBOSE = _IMP_saxs.IMP_VERBOSE

IMP_MEMORY = _IMP_saxs.IMP_MEMORY

IMP_NONE = _IMP_saxs.IMP_NONE

IMP_USAGE = _IMP_saxs.IMP_USAGE

IMP_INTERNAL = _IMP_saxs.IMP_INTERNAL

IMP_KERNEL_HAS_LOG4CXX = _IMP_saxs.IMP_KERNEL_HAS_LOG4CXX

IMP_COMPILER_HAS_AUTO = _IMP_saxs.IMP_COMPILER_HAS_AUTO

IMP_COMPILER_HAS_DEBUG_VECTOR = _IMP_saxs.IMP_COMPILER_HAS_DEBUG_VECTOR

IMP_COMPILER_HAS_UNIQUE_PTR = _IMP_saxs.IMP_COMPILER_HAS_UNIQUE_PTR

IMP_KERNEL_HAS_BOOST_RANDOM = _IMP_saxs.IMP_KERNEL_HAS_BOOST_RANDOM

IMP_KERNEL_HAS_NUMPY = _IMP_saxs.IMP_KERNEL_HAS_NUMPY

IMP_KERNEL_HAS_GPERFTOOLS = _IMP_saxs.IMP_KERNEL_HAS_GPERFTOOLS

IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER = _IMP_saxs.IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER

IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER = _IMP_saxs.IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER

IMPKERNEL_SHOW_WARNINGS = _IMP_saxs.IMPKERNEL_SHOW_WARNINGS


import sys
class _DirectorObjects(object):
    """@internal Simple class to keep references to director objects
       to prevent premature deletion."""
    def __init__(self):
        self._objects = []
    def register(self, obj):
        """Take a reference to a director object; will only work for
           refcounted C++ classes"""
        if hasattr(obj, 'get_ref_count'):
            self._objects.append(obj)
    def cleanup(self):
        """Only drop our reference and allow cleanup by Python if no other
           Python references exist (we hold 3 references: one in self._objects,
           one in x, and one in the argument list for getrefcount) *and* no
           other C++ references exist (the Python object always holds one)"""
        objs = [x for x in self._objects if sys.getrefcount(x) > 3 \
                                         or x.get_ref_count() > 1]
# Do in two steps so the references are kept until the end of the
# function (deleting references may trigger a fresh call to this method)
        self._objects = objs
    def get_object_count(self):
        """Get number of director objects (useful for testing only)"""
        return len(self._objects)
_director_objects = _DirectorObjects()

class _ostream(object):
    r"""Proxy of C++ std::ostream class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def write(self, osa_buf):
        r"""write(_ostream self, char const * osa_buf)"""
        return _IMP_saxs._ostream_write(self, osa_buf)

# Register _ostream in _IMP_saxs:
_IMP_saxs._ostream_swigregister(_ostream)

IMP_COMPILER_HAS_OVERRIDE = _IMP_saxs.IMP_COMPILER_HAS_OVERRIDE

IMP_COMPILER_HAS_FINAL = _IMP_saxs.IMP_COMPILER_HAS_FINAL

IMP_HAS_NOEXCEPT = _IMP_saxs.IMP_HAS_NOEXCEPT

IMP_C_OPEN_BINARY = _IMP_saxs.IMP_C_OPEN_BINARY

import IMP
IMP_CGAL_HAS_BOOST_FILESYSTEM = _IMP_saxs.IMP_CGAL_HAS_BOOST_FILESYSTEM

IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS = _IMP_saxs.IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS

IMP_CGAL_HAS_BOOST_RANDOM = _IMP_saxs.IMP_CGAL_HAS_BOOST_RANDOM

IMP_CGAL_HAS_BOOST_SYSTEM = _IMP_saxs.IMP_CGAL_HAS_BOOST_SYSTEM

IMP_CGAL_HAS_NUMPY = _IMP_saxs.IMP_CGAL_HAS_NUMPY

IMPCGAL_SHOW_WARNINGS = _IMP_saxs.IMPCGAL_SHOW_WARNINGS

import IMP.cgal
IMP_ALGEBRA_HAS_IMP_CGAL = _IMP_saxs.IMP_ALGEBRA_HAS_IMP_CGAL

IMP_ALGEBRA_HAS_BOOST_FILESYSTEM = _IMP_saxs.IMP_ALGEBRA_HAS_BOOST_FILESYSTEM

IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS = _IMP_saxs.IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS

IMP_ALGEBRA_HAS_BOOST_RANDOM = _IMP_saxs.IMP_ALGEBRA_HAS_BOOST_RANDOM

IMP_ALGEBRA_HAS_BOOST_SYSTEM = _IMP_saxs.IMP_ALGEBRA_HAS_BOOST_SYSTEM

IMP_ALGEBRA_HAS_CGAL = _IMP_saxs.IMP_ALGEBRA_HAS_CGAL

IMP_ALGEBRA_HAS_NUMPY = _IMP_saxs.IMP_ALGEBRA_HAS_NUMPY

IMP_ALGEBRA_HAS_ANN = _IMP_saxs.IMP_ALGEBRA_HAS_ANN

IMPALGEBRA_SHOW_WARNINGS = _IMP_saxs.IMPALGEBRA_SHOW_WARNINGS

import IMP.algebra
IMP_DISPLAY_HAS_IMP_CGAL = _IMP_saxs.IMP_DISPLAY_HAS_IMP_CGAL

IMP_DISPLAY_HAS_BOOST_FILESYSTEM = _IMP_saxs.IMP_DISPLAY_HAS_BOOST_FILESYSTEM

IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS = _IMP_saxs.IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS

IMP_DISPLAY_HAS_BOOST_RANDOM = _IMP_saxs.IMP_DISPLAY_HAS_BOOST_RANDOM

IMP_DISPLAY_HAS_BOOST_SYSTEM = _IMP_saxs.IMP_DISPLAY_HAS_BOOST_SYSTEM

IMP_DISPLAY_HAS_CGAL = _IMP_saxs.IMP_DISPLAY_HAS_CGAL

IMP_DISPLAY_HAS_NUMPY = _IMP_saxs.IMP_DISPLAY_HAS_NUMPY

IMPDISPLAY_SHOW_WARNINGS = _IMP_saxs.IMPDISPLAY_SHOW_WARNINGS

import IMP.display
IMP_SCORE_FUNCTOR_HAS_IMP_CGAL = _IMP_saxs.IMP_SCORE_FUNCTOR_HAS_IMP_CGAL

IMP_SCORE_FUNCTOR_HAS_BOOST_FILESYSTEM = _IMP_saxs.IMP_SCORE_FUNCTOR_HAS_BOOST_FILESYSTEM

IMP_SCORE_FUNCTOR_HAS_BOOST_PROGRAMOPTIONS = _IMP_saxs.IMP_SCORE_FUNCTOR_HAS_BOOST_PROGRAMOPTIONS

IMP_SCORE_FUNCTOR_HAS_BOOST_RANDOM = _IMP_saxs.IMP_SCORE_FUNCTOR_HAS_BOOST_RANDOM

IMP_SCORE_FUNCTOR_HAS_BOOST_SYSTEM = _IMP_saxs.IMP_SCORE_FUNCTOR_HAS_BOOST_SYSTEM

IMP_SCORE_FUNCTOR_HAS_CGAL = _IMP_saxs.IMP_SCORE_FUNCTOR_HAS_CGAL

IMP_SCORE_FUNCTOR_HAS_HDF5 = _IMP_saxs.IMP_SCORE_FUNCTOR_HAS_HDF5

IMP_SCORE_FUNCTOR_HAS_NUMPY = _IMP_saxs.IMP_SCORE_FUNCTOR_HAS_NUMPY

IMPSCOREFUNCTOR_SHOW_WARNINGS = _IMP_saxs.IMPSCOREFUNCTOR_SHOW_WARNINGS

import IMP.score_functor
IMP_CORE_HAS_IMP_CGAL = _IMP_saxs.IMP_CORE_HAS_IMP_CGAL

IMP_CORE_HAS_IMP_KERNEL = _IMP_saxs.IMP_CORE_HAS_IMP_KERNEL

IMP_CORE_HAS_BOOST_FILESYSTEM = _IMP_saxs.IMP_CORE_HAS_BOOST_FILESYSTEM

IMP_CORE_HAS_BOOST_PROGRAMOPTIONS = _IMP_saxs.IMP_CORE_HAS_BOOST_PROGRAMOPTIONS

IMP_CORE_HAS_BOOST_RANDOM = _IMP_saxs.IMP_CORE_HAS_BOOST_RANDOM

IMP_CORE_HAS_BOOST_SYSTEM = _IMP_saxs.IMP_CORE_HAS_BOOST_SYSTEM

IMP_CORE_HAS_CGAL = _IMP_saxs.IMP_CORE_HAS_CGAL

IMP_CORE_HAS_HDF5 = _IMP_saxs.IMP_CORE_HAS_HDF5

IMP_CORE_HAS_NUMPY = _IMP_saxs.IMP_CORE_HAS_NUMPY

IMPCORE_SHOW_WARNINGS = _IMP_saxs.IMPCORE_SHOW_WARNINGS

import IMP.core
IMP_CONTAINER_HAS_IMP_ALGEBRA = _IMP_saxs.IMP_CONTAINER_HAS_IMP_ALGEBRA

IMP_CONTAINER_HAS_IMP_CGAL = _IMP_saxs.IMP_CONTAINER_HAS_IMP_CGAL

IMP_CONTAINER_HAS_IMP_DISPLAY = _IMP_saxs.IMP_CONTAINER_HAS_IMP_DISPLAY

IMP_CONTAINER_HAS_IMP_KERNEL = _IMP_saxs.IMP_CONTAINER_HAS_IMP_KERNEL

IMP_CONTAINER_HAS_IMP_SCORE_FUNCTOR = _IMP_saxs.IMP_CONTAINER_HAS_IMP_SCORE_FUNCTOR

IMP_CONTAINER_HAS_BOOST_FILESYSTEM = _IMP_saxs.IMP_CONTAINER_HAS_BOOST_FILESYSTEM

IMP_CONTAINER_HAS_BOOST_PROGRAMOPTIONS = _IMP_saxs.IMP_CONTAINER_HAS_BOOST_PROGRAMOPTIONS

IMP_CONTAINER_HAS_BOOST_RANDOM = _IMP_saxs.IMP_CONTAINER_HAS_BOOST_RANDOM

IMP_CONTAINER_HAS_BOOST_SYSTEM = _IMP_saxs.IMP_CONTAINER_HAS_BOOST_SYSTEM

IMP_CONTAINER_HAS_CGAL = _IMP_saxs.IMP_CONTAINER_HAS_CGAL

IMP_CONTAINER_HAS_HDF5 = _IMP_saxs.IMP_CONTAINER_HAS_HDF5

IMP_CONTAINER_HAS_NUMPY = _IMP_saxs.IMP_CONTAINER_HAS_NUMPY

IMP_CONTAINER_HAS_PYTHON_IHM = _IMP_saxs.IMP_CONTAINER_HAS_PYTHON_IHM

IMP_CONTAINER_HAS_GOOGLE_DENSE_HASH_MAP = _IMP_saxs.IMP_CONTAINER_HAS_GOOGLE_DENSE_HASH_MAP

IMP_CONTAINER_HAS_ROBIN_MAP = _IMP_saxs.IMP_CONTAINER_HAS_ROBIN_MAP

IMPCONTAINER_SHOW_WARNINGS = _IMP_saxs.IMPCONTAINER_SHOW_WARNINGS

import IMP.container
IMP_ATOM_HAS_IMP_CGAL = _IMP_saxs.IMP_ATOM_HAS_IMP_CGAL

IMP_ATOM_HAS_IMP_KERNEL = _IMP_saxs.IMP_ATOM_HAS_IMP_KERNEL

IMP_ATOM_HAS_IMP_SCORE_FUNCTOR = _IMP_saxs.IMP_ATOM_HAS_IMP_SCORE_FUNCTOR

IMP_ATOM_HAS_BOOST_PROGRAMOPTIONS = _IMP_saxs.IMP_ATOM_HAS_BOOST_PROGRAMOPTIONS

IMP_ATOM_HAS_BOOST_RANDOM = _IMP_saxs.IMP_ATOM_HAS_BOOST_RANDOM

IMP_ATOM_HAS_BOOST_REGEX = _IMP_saxs.IMP_ATOM_HAS_BOOST_REGEX

IMP_ATOM_HAS_BOOST_SYSTEM = _IMP_saxs.IMP_ATOM_HAS_BOOST_SYSTEM

IMP_ATOM_HAS_CGAL = _IMP_saxs.IMP_ATOM_HAS_CGAL

IMP_ATOM_HAS_HDF5 = _IMP_saxs.IMP_ATOM_HAS_HDF5

IMP_ATOM_HAS_NUMPY = _IMP_saxs.IMP_ATOM_HAS_NUMPY

IMP_ATOM_HAS_PYTHON_IHM = _IMP_saxs.IMP_ATOM_HAS_PYTHON_IHM

IMPATOM_SHOW_WARNINGS = _IMP_saxs.IMPATOM_SHOW_WARNINGS

IMP_ATOM_TYPE_INDEX = _IMP_saxs.IMP_ATOM_TYPE_INDEX

IMP_RESIDUE_TYPE_INDEX = _IMP_saxs.IMP_RESIDUE_TYPE_INDEX

IMP_HIERARCHY_TYPE_INDEX = _IMP_saxs.IMP_HIERARCHY_TYPE_INDEX

IMP_CHAIN_TYPE_INDEX = _IMP_saxs.IMP_CHAIN_TYPE_INDEX

import IMP.atom
IMP_SAXS_HAS_IMP_CGAL = _IMP_saxs.IMP_SAXS_HAS_IMP_CGAL

IMP_SAXS_HAS_IMP_CONTAINER = _IMP_saxs.IMP_SAXS_HAS_IMP_CONTAINER

IMP_SAXS_HAS_IMP_DISPLAY = _IMP_saxs.IMP_SAXS_HAS_IMP_DISPLAY

IMP_SAXS_HAS_IMP_KERNEL = _IMP_saxs.IMP_SAXS_HAS_IMP_KERNEL

IMP_SAXS_HAS_IMP_SCORE_FUNCTOR = _IMP_saxs.IMP_SAXS_HAS_IMP_SCORE_FUNCTOR

IMP_SAXS_HAS_BOOST_FILESYSTEM = _IMP_saxs.IMP_SAXS_HAS_BOOST_FILESYSTEM

IMP_SAXS_HAS_BOOST_PROGRAMOPTIONS = _IMP_saxs.IMP_SAXS_HAS_BOOST_PROGRAMOPTIONS

IMP_SAXS_HAS_BOOST_RANDOM = _IMP_saxs.IMP_SAXS_HAS_BOOST_RANDOM

IMP_SAXS_HAS_BOOST_REGEX = _IMP_saxs.IMP_SAXS_HAS_BOOST_REGEX

IMP_SAXS_HAS_BOOST_SYSTEM = _IMP_saxs.IMP_SAXS_HAS_BOOST_SYSTEM

IMP_SAXS_HAS_CGAL = _IMP_saxs.IMP_SAXS_HAS_CGAL

IMP_SAXS_HAS_HDF5 = _IMP_saxs.IMP_SAXS_HAS_HDF5

IMP_SAXS_HAS_NUMPY = _IMP_saxs.IMP_SAXS_HAS_NUMPY

IMP_SAXS_HAS_PYTHON_IHM = _IMP_saxs.IMP_SAXS_HAS_PYTHON_IHM

IMPSAXS_SHOW_WARNINGS = _IMP_saxs.IMPSAXS_SHOW_WARNINGS

class DistBase(object):
    r"""Proxy of C++ std::vector< double > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def iterator(self):
        r"""iterator(DistBase self) -> IMP_SAXS_SwigPyIterator"""
        return _IMP_saxs.DistBase_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        r"""__nonzero__(DistBase self) -> bool"""
        return _IMP_saxs.DistBase___nonzero__(self)

    def __bool__(self):
        r"""__bool__(DistBase self) -> bool"""
        return _IMP_saxs.DistBase___bool__(self)

    def __len__(self):
        r"""__len__(DistBase self) -> std::vector< double >::size_type"""
        return _IMP_saxs.DistBase___len__(self)

    def __getslice__(self, i, j):
        r"""__getslice__(DistBase self, std::vector< double >::difference_type i, std::vector< double >::difference_type j) -> DistBase"""
        return _IMP_saxs.DistBase___getslice__(self, i, j)

    def __setslice__(self, *args):
        r"""
        __setslice__(DistBase self, std::vector< double >::difference_type i, std::vector< double >::difference_type j)
        __setslice__(DistBase self, std::vector< double >::difference_type i, std::vector< double >::difference_type j, DistBase v)
        """
        return _IMP_saxs.DistBase___setslice__(self, *args)

    def __delslice__(self, i, j):
        r"""__delslice__(DistBase self, std::vector< double >::difference_type i, std::vector< double >::difference_type j)"""
        return _IMP_saxs.DistBase___delslice__(self, i, j)

    def __delitem__(self, *args):
        r"""
        __delitem__(DistBase self, std::vector< double >::difference_type i)
        __delitem__(DistBase self, PySliceObject * slice)
        """
        return _IMP_saxs.DistBase___delitem__(self, *args)

    def __getitem__(self, *args):
        r"""
        __getitem__(DistBase self, PySliceObject * slice) -> DistBase
        __getitem__(DistBase self, std::vector< double >::difference_type i) -> std::vector< double >::value_type const &
        """
        return _IMP_saxs.DistBase___getitem__(self, *args)

    def __setitem__(self, *args):
        r"""
        __setitem__(DistBase self, PySliceObject * slice, DistBase v)
        __setitem__(DistBase self, PySliceObject * slice)
        __setitem__(DistBase self, std::vector< double >::difference_type i, std::vector< double >::value_type const & x)
        """
        return _IMP_saxs.DistBase___setitem__(self, *args)

    def pop(self):
        r"""pop(DistBase self) -> std::vector< double >::value_type"""
        return _IMP_saxs.DistBase_pop(self)

    def append(self, x):
        r"""append(DistBase self, std::vector< double >::value_type const & x)"""
        return _IMP_saxs.DistBase_append(self, x)

    def empty(self):
        r"""empty(DistBase self) -> bool"""
        return _IMP_saxs.DistBase_empty(self)

    def size(self):
        r"""size(DistBase self) -> std::vector< double >::size_type"""
        return _IMP_saxs.DistBase_size(self)

    def swap(self, v):
        r"""swap(DistBase self, DistBase v)"""
        return _IMP_saxs.DistBase_swap(self, v)

    def begin(self):
        r"""begin(DistBase self) -> std::vector< double >::iterator"""
        return _IMP_saxs.DistBase_begin(self)

    def end(self):
        r"""end(DistBase self) -> std::vector< double >::iterator"""
        return _IMP_saxs.DistBase_end(self)

    def rbegin(self):
        r"""rbegin(DistBase self) -> std::vector< double >::reverse_iterator"""
        return _IMP_saxs.DistBase_rbegin(self)

    def rend(self):
        r"""rend(DistBase self) -> std::vector< double >::reverse_iterator"""
        return _IMP_saxs.DistBase_rend(self)

    def clear(self):
        r"""clear(DistBase self)"""
        return _IMP_saxs.DistBase_clear(self)

    def get_allocator(self):
        r"""get_allocator(DistBase self) -> std::vector< double >::allocator_type"""
        return _IMP_saxs.DistBase_get_allocator(self)

    def pop_back(self):
        r"""pop_back(DistBase self)"""
        return _IMP_saxs.DistBase_pop_back(self)

    def erase(self, *args):
        r"""
        erase(DistBase self, std::vector< double >::iterator pos) -> std::vector< double >::iterator
        erase(DistBase self, std::vector< double >::iterator first, std::vector< double >::iterator last) -> std::vector< double >::iterator
        """
        return _IMP_saxs.DistBase_erase(self, *args)

    def __init__(self, *args):
        r"""
        __init__(DistBase self) -> DistBase
        __init__(DistBase self, DistBase other) -> DistBase
        __init__(DistBase self, std::vector< double >::size_type size) -> DistBase
        __init__(DistBase self, std::vector< double >::size_type size, std::vector< double >::value_type const & value) -> DistBase
        """
        _IMP_saxs.DistBase_swiginit(self, _IMP_saxs.new_DistBase(*args))

    def push_back(self, x):
        r"""push_back(DistBase self, std::vector< double >::value_type const & x)"""
        return _IMP_saxs.DistBase_push_back(self, x)

    def front(self):
        r"""front(DistBase self) -> std::vector< double >::value_type const &"""
        return _IMP_saxs.DistBase_front(self)

    def back(self):
        r"""back(DistBase self) -> std::vector< double >::value_type const &"""
        return _IMP_saxs.DistBase_back(self)

    def assign(self, n, x):
        r"""assign(DistBase self, std::vector< double >::size_type n, std::vector< double >::value_type const & x)"""
        return _IMP_saxs.DistBase_assign(self, n, x)

    def resize(self, *args):
        r"""
        resize(DistBase self, std::vector< double >::size_type new_size)
        resize(DistBase self, std::vector< double >::size_type new_size, std::vector< double >::value_type const & x)
        """
        return _IMP_saxs.DistBase_resize(self, *args)

    def insert(self, *args):
        r"""
        insert(DistBase self, std::vector< double >::iterator pos, std::vector< double >::value_type const & x) -> std::vector< double >::iterator
        insert(DistBase self, std::vector< double >::iterator pos, std::vector< double >::size_type n, std::vector< double >::value_type const & x)
        """
        return _IMP_saxs.DistBase_insert(self, *args)

    def reserve(self, n):
        r"""reserve(DistBase self, std::vector< double >::size_type n)"""
        return _IMP_saxs.DistBase_reserve(self, n)

    def capacity(self):
        r"""capacity(DistBase self) -> std::vector< double >::size_type"""
        return _IMP_saxs.DistBase_capacity(self)
    __swig_destroy__ = _IMP_saxs.delete_DistBase

# Register DistBase in _IMP_saxs:
_IMP_saxs.DistBase_swigregister(DistBase)


_object_types.append("Profile")


def _object_cast_to_Profile(o):
    r"""_object_cast_to_Profile(Object o) -> Profile"""
    return _IMP_saxs._object_cast_to_Profile(o)

_object_types.append("ChiScore")


def _object_cast_to_ChiScore(o):
    r"""_object_cast_to_ChiScore(Object o) -> ChiScore"""
    return _IMP_saxs._object_cast_to_ChiScore(o)

_object_types.append("ChiScoreLog")


def _object_cast_to_ChiScoreLog(o):
    r"""_object_cast_to_ChiScoreLog(Object o) -> ChiScoreLog"""
    return _IMP_saxs._object_cast_to_ChiScoreLog(o)
ALL_ATOMS = _IMP_saxs.ALL_ATOMS

HEAVY_ATOMS = _IMP_saxs.HEAVY_ATOMS

CA_ATOMS = _IMP_saxs.CA_ATOMS

RESIDUES = _IMP_saxs.RESIDUES

class FormFactorTable(object):
    r"""Proxy of C++ IMP::saxs::FormFactorTable class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(FormFactorTable self) -> FormFactorTable
        __init__(FormFactorTable self, std::string const & table_name, double min_q, double max_q, double delta_q) -> FormFactorTable
        """
        _IMP_saxs.FormFactorTable_swiginit(self, _IMP_saxs.new_FormFactorTable(*args))

    def get_form_factor(self, *args):
        r"""get_form_factor(FormFactorTable self, Particle p, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> double"""
        return _IMP_saxs.FormFactorTable_get_form_factor(self, *args)

    def get_vacuum_form_factor(self, *args):
        r"""get_vacuum_form_factor(FormFactorTable self, Particle p, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> double"""
        return _IMP_saxs.FormFactorTable_get_vacuum_form_factor(self, *args)

    def get_dummy_form_factor(self, *args):
        r"""get_dummy_form_factor(FormFactorTable self, Particle p, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> double"""
        return _IMP_saxs.FormFactorTable_get_dummy_form_factor(self, *args)

    def get_water_form_factor(self):
        r"""get_water_form_factor(FormFactorTable self) -> double"""
        return _IMP_saxs.FormFactorTable_get_water_form_factor(self)

    def get_vacuum_water_form_factor(self):
        r"""get_vacuum_water_form_factor(FormFactorTable self) -> double"""
        return _IMP_saxs.FormFactorTable_get_vacuum_water_form_factor(self)

    def get_dummy_water_form_factor(self):
        r"""get_dummy_water_form_factor(FormFactorTable self) -> double"""
        return _IMP_saxs.FormFactorTable_get_dummy_water_form_factor(self)

    def get_form_factors(self, *args):
        r"""get_form_factors(FormFactorTable self, Particle p, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> IMP::Vector< double > const"""
        return _IMP_saxs.FormFactorTable_get_form_factors(self, *args)

    def get_vacuum_form_factors(self, *args):
        r"""get_vacuum_form_factors(FormFactorTable self, Particle p, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> IMP::Vector< double > const"""
        return _IMP_saxs.FormFactorTable_get_vacuum_form_factors(self, *args)

    def get_dummy_form_factors(self, *args):
        r"""get_dummy_form_factors(FormFactorTable self, Particle p, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> IMP::Vector< double > const"""
        return _IMP_saxs.FormFactorTable_get_dummy_form_factors(self, *args)

    def get_water_form_factors(self):
        r"""get_water_form_factors(FormFactorTable self) -> IMP::Vector< double > const &"""
        return _IMP_saxs.FormFactorTable_get_water_form_factors(self)

    def get_water_vacuum_form_factors(self):
        r"""get_water_vacuum_form_factors(FormFactorTable self) -> IMP::Vector< double > const &"""
        return _IMP_saxs.FormFactorTable_get_water_vacuum_form_factors(self)

    def get_water_dummy_form_factors(self):
        r"""get_water_dummy_form_factors(FormFactorTable self) -> IMP::Vector< double > const &"""
        return _IMP_saxs.FormFactorTable_get_water_dummy_form_factors(self)

    def get_radius(self, *args):
        r"""get_radius(FormFactorTable self, Particle p, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> double"""
        return _IMP_saxs.FormFactorTable_get_radius(self, *args)

    def get_volume(self, *args):
        r"""get_volume(FormFactorTable self, Particle p, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> double"""
        return _IMP_saxs.FormFactorTable_get_volume(self, *args)

    def show(self, *args):
        r"""show(FormFactorTable self, _ostream out=std::cout, std::string prefix="")"""
        return _IMP_saxs.FormFactorTable_show(self, *args)
    rho_ = property(_IMP_saxs.FormFactorTable_rho__get, _IMP_saxs.FormFactorTable_rho__set, doc=r"""rho_ : double""")
    __swig_destroy__ = _IMP_saxs.delete_FormFactorTable

# Register FormFactorTable in _IMP_saxs:
_IMP_saxs.FormFactorTable_swigregister(FormFactorTable)
cvar = _IMP_saxs.cvar


def get_default_form_factor_table():
    r"""get_default_form_factor_table() -> FormFactorTable"""
    return _IMP_saxs.get_default_form_factor_table()
class Profile(IMP.Object):
    r"""Proxy of C++ IMP::saxs::Profile class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(Profile self, std::string const & file_name, bool fit_file=False, double max_q=0.0, int units=1) -> Profile
        __init__(Profile self, double qmin=0.0, double qmax=0.5, double delta=0.005) -> Profile
        """
        _IMP_saxs.Profile_swiginit(self, _IMP_saxs.new_Profile(*args))

    def calculate_profile_partial(self, *args):
        r"""
        calculate_profile_partial(Profile self, IMP::Particles const & particles, IMP::Vector< double > const & surface=IMP::Vector< double >(), IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS)
        calculate_profile_partial(Profile self, IMP::Particles const & particles1, IMP::Particles const & particles2, IMP::Vector< double > const & surface1=IMP::Vector< double >(), IMP::Vector< double > const & surface2=IMP::Vector< double >(), IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS)
        """
        return _IMP_saxs.Profile_calculate_profile_partial(self, *args)

    def calculate_profile_reciprocal_partial(self, *args):
        r"""calculate_profile_reciprocal_partial(Profile self, IMP::Particles const & particles, IMP::Vector< double > const & surface=IMP::Vector< double >(), IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS)"""
        return _IMP_saxs.Profile_calculate_profile_reciprocal_partial(self, *args)

    def calculate_profile(self, *args):
        r"""
        calculate_profile(Profile self, IMP::Particles const & particles, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS, bool reciprocal=False)
        calculate_profile(Profile self, IMP::Particles const & particles1, IMP::Particles const & particles2, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS)
        """
        return _IMP_saxs.Profile_calculate_profile(self, *args)

    def calculate_I0(self, *args):
        r"""calculate_I0(Profile self, IMP::Particles const & particles, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> double"""
        return _IMP_saxs.Profile_calculate_I0(self, *args)

    def calculate_profile_constant_form_factor(self, particles, form_factor=1.0):
        r"""calculate_profile_constant_form_factor(Profile self, IMP::Particles const & particles, double form_factor=1.0)"""
        return _IMP_saxs.Profile_calculate_profile_constant_form_factor(self, particles, form_factor)

    def calculate_profile_symmetric(self, *args):
        r"""calculate_profile_symmetric(Profile self, IMP::Particles const & particles, unsigned int n, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS)"""
        return _IMP_saxs.Profile_calculate_profile_symmetric(self, *args)

    def profile_2_distribution(self, rd, max_distance):
        r"""profile_2_distribution(Profile self, RadialDistributionFunction rd, double max_distance)"""
        return _IMP_saxs.Profile_profile_2_distribution(self, rd, max_distance)

    def distribution_2_profile(self, r_dist):
        r"""distribution_2_profile(Profile self, RadialDistributionFunction r_dist)"""
        return _IMP_saxs.Profile_distribution_2_profile(self, r_dist)

    def resample(self, exp_profile, resampled_profile):
        r"""resample(Profile self, Profile exp_profile, Profile resampled_profile)"""
        return _IMP_saxs.Profile_resample(self, exp_profile, resampled_profile)

    def downsample(self, downsampled_profile, point_number):
        r"""downsample(Profile self, Profile downsampled_profile, unsigned int point_number)"""
        return _IMP_saxs.Profile_downsample(self, downsampled_profile, point_number)

    def radius_of_gyration(self, end_q_rg=1.3):
        r"""radius_of_gyration(Profile self, double end_q_rg=1.3) -> double"""
        return _IMP_saxs.Profile_radius_of_gyration(self, end_q_rg)

    def mean_intensity(self):
        r"""mean_intensity(Profile self) -> double"""
        return _IMP_saxs.Profile_mean_intensity(self)

    def read_SAXS_file(self, file_name, fit_file=False, max_q=0.0, units=1):
        r"""read_SAXS_file(Profile self, std::string const & file_name, bool fit_file=False, double max_q=0.0, int units=1)"""
        return _IMP_saxs.Profile_read_SAXS_file(self, file_name, fit_file, max_q, units)

    def write_SAXS_file(self, file_name, max_q=0.0):
        r"""write_SAXS_file(Profile self, std::string const & file_name, double max_q=0.0)"""
        return _IMP_saxs.Profile_write_SAXS_file(self, file_name, max_q)

    def read_partial_profiles(self, file_name):
        r"""read_partial_profiles(Profile self, std::string const & file_name)"""
        return _IMP_saxs.Profile_read_partial_profiles(self, file_name)

    def write_partial_profiles(self, file_name):
        r"""write_partial_profiles(Profile self, std::string const & file_name)"""
        return _IMP_saxs.Profile_write_partial_profiles(self, file_name)

    def get_delta_q(self):
        r"""get_delta_q(Profile self) -> double"""
        return _IMP_saxs.Profile_get_delta_q(self)

    def get_min_q(self):
        r"""get_min_q(Profile self) -> double"""
        return _IMP_saxs.Profile_get_min_q(self)

    def get_max_q(self):
        r"""get_max_q(Profile self) -> double"""
        return _IMP_saxs.Profile_get_max_q(self)

    def get_intensity(self, i):
        r"""get_intensity(Profile self, unsigned int i) -> double"""
        return _IMP_saxs.Profile_get_intensity(self, i)

    def get_q(self, i):
        r"""get_q(Profile self, unsigned int i) -> double"""
        return _IMP_saxs.Profile_get_q(self, i)

    def get_error(self, i):
        r"""get_error(Profile self, unsigned int i) -> double"""
        return _IMP_saxs.Profile_get_error(self, i)

    def get_weight(self, i):
        r"""get_weight(Profile self, unsigned int i) -> double"""
        return _IMP_saxs.Profile_get_weight(self, i)

    def get_qs(self):
        r"""get_qs(Profile self) -> Eigen::VectorXf const &"""
        return _IMP_saxs.Profile_get_qs(self)

    def get_intensities(self):
        r"""get_intensities(Profile self) -> Eigen::VectorXf const &"""
        return _IMP_saxs.Profile_get_intensities(self)

    def get_errors(self):
        r"""get_errors(Profile self) -> Eigen::VectorXf const &"""
        return _IMP_saxs.Profile_get_errors(self)

    def get_average_radius(self):
        r"""get_average_radius(Profile self) -> double"""
        return _IMP_saxs.Profile_get_average_radius(self)

    def size(self):
        r"""size(Profile self) -> unsigned int"""
        return _IMP_saxs.Profile_size(self)

    def is_uniform_sampling(self):
        r"""is_uniform_sampling(Profile self) -> bool"""
        return _IMP_saxs.Profile_is_uniform_sampling(self)

    def is_partial_profile(self):
        r"""is_partial_profile(Profile self) -> bool"""
        return _IMP_saxs.Profile_is_partial_profile(self)

    def get_name(self):
        r"""get_name(Profile self) -> std::string"""
        return _IMP_saxs.Profile_get_name(self)

    def get_id(self):
        r"""get_id(Profile self) -> unsigned int"""
        return _IMP_saxs.Profile_get_id(self)

    def set_qs(self, q):
        r"""set_qs(Profile self, Eigen::VectorXf const & q)"""
        return _IMP_saxs.Profile_set_qs(self, q)

    def set_intensities(self, i):
        r"""set_intensities(Profile self, Eigen::VectorXf const & i)"""
        return _IMP_saxs.Profile_set_intensities(self, i)

    def set_errors(self, e):
        r"""set_errors(Profile self, Eigen::VectorXf const & e)"""
        return _IMP_saxs.Profile_set_errors(self, e)

    def set_intensity(self, i, iq):
        r"""set_intensity(Profile self, unsigned int i, double iq)"""
        return _IMP_saxs.Profile_set_intensity(self, i, iq)

    def set_ff_table(self, ff_table):
        r"""set_ff_table(Profile self, FormFactorTable ff_table)"""
        return _IMP_saxs.Profile_set_ff_table(self, ff_table)

    def set_average_radius(self, r):
        r"""set_average_radius(Profile self, double r)"""
        return _IMP_saxs.Profile_set_average_radius(self, r)

    def set_average_volume(self, v):
        r"""set_average_volume(Profile self, double v)"""
        return _IMP_saxs.Profile_set_average_volume(self, v)

    def set_name(self, name):
        r"""set_name(Profile self, std::string name)"""
        return _IMP_saxs.Profile_set_name(self, name)

    def set_id(self, id):
        r"""set_id(Profile self, unsigned int id)"""
        return _IMP_saxs.Profile_set_id(self, id)

    def set_beam_profile(self, beam_profile_file):
        r"""set_beam_profile(Profile self, std::string beam_profile_file)"""
        return _IMP_saxs.Profile_set_beam_profile(self, beam_profile_file)

    def add_errors(self):
        r"""add_errors(Profile self)"""
        return _IMP_saxs.Profile_add_errors(self)

    def add_noise(self, percentage=0.03):
        r"""add_noise(Profile self, double percentage=0.03)"""
        return _IMP_saxs.Profile_add_noise(self, percentage)

    def sum_partial_profiles(self, c1, c2, check_cashed=True):
        r"""sum_partial_profiles(Profile self, double c1, double c2, bool check_cashed=True)"""
        return _IMP_saxs.Profile_sum_partial_profiles(self, c1, c2, check_cashed)

    def add(self, *args):
        r"""
        add(Profile self, Profile other_profile, double weight=1.0)
        add(Profile self, IMP::Vector< IMP::saxs::Profile * > const & profiles, IMP::Vector< double > const & weights=IMP::Vector< double >())
        """
        return _IMP_saxs.Profile_add(self, *args)

    def add_partial_profiles(self, *args):
        r"""
        add_partial_profiles(Profile self, Profile other_profile, double weight=1.0)
        add_partial_profiles(Profile self, IMP::Vector< IMP::saxs::Profile * > const & profiles, IMP::Vector< double > const & weights=IMP::Vector< double >())
        """
        return _IMP_saxs.Profile_add_partial_profiles(self, *args)

    def background_adjust(self, start_q):
        r"""background_adjust(Profile self, double start_q)"""
        return _IMP_saxs.Profile_background_adjust(self, start_q)

    def scale(self, c):
        r"""scale(Profile self, double c)"""
        return _IMP_saxs.Profile_scale(self, c)

    def offset(self, c):
        r"""offset(Profile self, double c)"""
        return _IMP_saxs.Profile_offset(self, c)

    def copy_errors(self, exp_profile):
        r"""copy_errors(Profile self, Profile exp_profile)"""
        return _IMP_saxs.Profile_copy_errors(self, exp_profile)

    def get_version_info(self):
        r"""get_version_info(Profile self) -> VersionInfo"""
        return _IMP_saxs.Profile_get_version_info(self)

    def __str__(self):
        r"""__str__(Profile self) -> std::string"""
        return _IMP_saxs.Profile___str__(self)

    def __repr__(self):
        r"""__repr__(Profile self) -> std::string"""
        return _IMP_saxs.Profile___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_Profile(o)


# Register Profile in _IMP_saxs:
_IMP_saxs.Profile_swigregister(Profile)
Profile.modulation_function_parameter_ = _IMP_saxs.cvar.Profile_modulation_function_parameter_

class FitParameters(object):
    r"""Proxy of C++ IMP::saxs::FitParameters class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(FitParameters self) -> FitParameters
        __init__(FitParameters self, double chi_square, double c1, double c2, double c, double o) -> FitParameters
        __init__(FitParameters self, double chi_square, double c1, double c2) -> FitParameters
        """
        _IMP_saxs.FitParameters_swiginit(self, _IMP_saxs.new_FitParameters(*args))

    def get_score(self):
        r"""get_score(FitParameters self) -> double"""
        return _IMP_saxs.FitParameters_get_score(self)

    def get_chi_square(self):
        r"""get_chi_square(FitParameters self) -> double"""
        return _IMP_saxs.FitParameters_get_chi_square(self)

    def get_c1(self):
        r"""get_c1(FitParameters self) -> double"""
        return _IMP_saxs.FitParameters_get_c1(self)

    def get_c2(self):
        r"""get_c2(FitParameters self) -> double"""
        return _IMP_saxs.FitParameters_get_c2(self)

    def get_scale(self):
        r"""get_scale(FitParameters self) -> double"""
        return _IMP_saxs.FitParameters_get_scale(self)

    def get_offset(self):
        r"""get_offset(FitParameters self) -> double"""
        return _IMP_saxs.FitParameters_get_offset(self)

    def get_default_chi_square(self):
        r"""get_default_chi_square(FitParameters self) -> double"""
        return _IMP_saxs.FitParameters_get_default_chi_square(self)

    def get_pdb_file_name(self):
        r"""get_pdb_file_name(FitParameters self) -> std::string"""
        return _IMP_saxs.FitParameters_get_pdb_file_name(self)

    def get_profile_file_name(self):
        r"""get_profile_file_name(FitParameters self) -> std::string"""
        return _IMP_saxs.FitParameters_get_profile_file_name(self)

    def get_mol_index(self):
        r"""get_mol_index(FitParameters self) -> int"""
        return _IMP_saxs.FitParameters_get_mol_index(self)

    def set_score(self, score):
        r"""set_score(FitParameters self, double score)"""
        return _IMP_saxs.FitParameters_set_score(self, score)

    def set_chi_square(self, chi_square):
        r"""set_chi_square(FitParameters self, double chi_square)"""
        return _IMP_saxs.FitParameters_set_chi_square(self, chi_square)

    def set_default_chi_square(self, chi_square):
        r"""set_default_chi_square(FitParameters self, double chi_square)"""
        return _IMP_saxs.FitParameters_set_default_chi_square(self, chi_square)

    def set_profile_file_name(self, file_name):
        r"""set_profile_file_name(FitParameters self, std::string file_name)"""
        return _IMP_saxs.FitParameters_set_profile_file_name(self, file_name)

    def set_pdb_file_name(self, file_name):
        r"""set_pdb_file_name(FitParameters self, std::string file_name)"""
        return _IMP_saxs.FitParameters_set_pdb_file_name(self, file_name)

    def set_mol_index(self, index):
        r"""set_mol_index(FitParameters self, int index)"""
        return _IMP_saxs.FitParameters_set_mol_index(self, index)

    def show(self, s):
        r"""show(FitParameters self, _ostream s)"""
        return _IMP_saxs.FitParameters_show(self, s)
    __swig_destroy__ = _IMP_saxs.delete_FitParameters

# Register FitParameters in _IMP_saxs:
_IMP_saxs.FitParameters_swigregister(FitParameters)

class WeightedFitParameters(FitParameters):
    r"""Proxy of C++ IMP::saxs::WeightedFitParameters class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(WeightedFitParameters self) -> WeightedFitParameters
        __init__(WeightedFitParameters self, double chi_square, double c1, double c2, IMP::Vector< double > const & weights) -> WeightedFitParameters
        """
        _IMP_saxs.WeightedFitParameters_swiginit(self, _IMP_saxs.new_WeightedFitParameters(*args))

    def get_weights(self):
        r"""get_weights(WeightedFitParameters self) -> IMP::Vector< double > const &"""
        return _IMP_saxs.WeightedFitParameters_get_weights(self)

    def set_weights(self, weights):
        r"""set_weights(WeightedFitParameters self, IMP::Vector< double > const & weights)"""
        return _IMP_saxs.WeightedFitParameters_set_weights(self, weights)

    def show(self, s):
        r"""show(WeightedFitParameters self, _ostream s)"""
        return _IMP_saxs.WeightedFitParameters_show(self, s)
    __swig_destroy__ = _IMP_saxs.delete_WeightedFitParameters

# Register WeightedFitParameters in _IMP_saxs:
_IMP_saxs.WeightedFitParameters_swigregister(WeightedFitParameters)

class ChiScore(IMP.Object):
    r"""Proxy of C++ IMP::saxs::ChiScore class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self):
        r"""__init__(ChiScore self) -> ChiScore"""
        _IMP_saxs.ChiScore_swiginit(self, _IMP_saxs.new_ChiScore())

    def compute_score(self, exp_profile, model_profile, use_offset=False):
        r"""compute_score(ChiScore self, Profile exp_profile, Profile model_profile, bool use_offset=False) -> double"""
        return _IMP_saxs.ChiScore_compute_score(self, exp_profile, model_profile, use_offset)

    def compute_scale_factor(self, exp_profile, model_profile, offset=0.0):
        r"""compute_scale_factor(ChiScore self, Profile exp_profile, Profile model_profile, double offset=0.0) -> double"""
        return _IMP_saxs.ChiScore_compute_scale_factor(self, exp_profile, model_profile, offset)

    def compute_offset(self, exp_profile, model_profile):
        r"""compute_offset(ChiScore self, Profile exp_profile, Profile model_profile) -> double"""
        return _IMP_saxs.ChiScore_compute_offset(self, exp_profile, model_profile)

    def get_version_info(self):
        r"""get_version_info(ChiScore self) -> VersionInfo"""
        return _IMP_saxs.ChiScore_get_version_info(self)

    def __str__(self):
        r"""__str__(ChiScore self) -> std::string"""
        return _IMP_saxs.ChiScore___str__(self)

    def __repr__(self):
        r"""__repr__(ChiScore self) -> std::string"""
        return _IMP_saxs.ChiScore___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ChiScore(o)


# Register ChiScore in _IMP_saxs:
_IMP_saxs.ChiScore_swigregister(ChiScore)

class ChiScoreLog(IMP.Object):
    r"""Proxy of C++ IMP::saxs::ChiScoreLog class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self):
        r"""__init__(ChiScoreLog self) -> ChiScoreLog"""
        _IMP_saxs.ChiScoreLog_swiginit(self, _IMP_saxs.new_ChiScoreLog())

    def compute_score(self, *args):
        r"""
        compute_score(ChiScoreLog self, Profile exp_profile, Profile model_profile, bool use_offset=False) -> double
        compute_score(ChiScoreLog self, Profile exp_profile, Profile model_profile, double min_q, double max_q) -> double
        """
        return _IMP_saxs.ChiScoreLog_compute_score(self, *args)

    def compute_scale_factor(self, exp_profile, model_profile, offset=0.0):
        r"""compute_scale_factor(ChiScoreLog self, Profile exp_profile, Profile model_profile, double offset=0.0) -> double"""
        return _IMP_saxs.ChiScoreLog_compute_scale_factor(self, exp_profile, model_profile, offset)

    def compute_offset(self, exp_profile, model_profile):
        r"""compute_offset(ChiScoreLog self, Profile exp_profile, Profile model_profile) -> double"""
        return _IMP_saxs.ChiScoreLog_compute_offset(self, exp_profile, model_profile)

    def get_version_info(self):
        r"""get_version_info(ChiScoreLog self) -> VersionInfo"""
        return _IMP_saxs.ChiScoreLog_get_version_info(self)

    def __str__(self):
        r"""__str__(ChiScoreLog self) -> std::string"""
        return _IMP_saxs.ChiScoreLog___str__(self)

    def __repr__(self):
        r"""__repr__(ChiScoreLog self) -> std::string"""
        return _IMP_saxs.ChiScoreLog___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ChiScoreLog(o)


# Register ChiScoreLog in _IMP_saxs:
_IMP_saxs.ChiScoreLog_swigregister(ChiScoreLog)

class ProfileFitterChi(IMP.Object):
    r"""Proxy of C++ IMP::saxs::ProfileFitter< IMP::saxs::ChiScore > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(ProfileFitterChi self, Profile exp_profile) -> ProfileFitterChi
        __init__(ProfileFitterChi self, Profile exp_profile, ChiScore sf) -> ProfileFitterChi
        """
        _IMP_saxs.ProfileFitterChi_swiginit(self, _IMP_saxs.new_ProfileFitterChi(*args))

    def compute_score(self, *args):
        r"""compute_score(ProfileFitterChi self, Profile model_profile, bool use_offset=False, std::string const fit_file_name="") -> double"""
        return _IMP_saxs.ProfileFitterChi_compute_score(self, *args)

    def fit_profile(self, *args):
        r"""fit_profile(ProfileFitterChi self, Profile partial_profile, double min_c1=0.95, double max_c1=1.05, double min_c2=-2.0, double max_c2=4.0, bool use_offset=False, std::string const fit_file_name="") -> FitParameters"""
        return _IMP_saxs.ProfileFitterChi_fit_profile(self, *args)

    def compute_scale_factor(self, model_profile, offset=0.0):
        r"""compute_scale_factor(ProfileFitterChi self, Profile model_profile, double offset=0.0) -> double"""
        return _IMP_saxs.ProfileFitterChi_compute_scale_factor(self, model_profile, offset)

    def compute_offset(self, model_profile):
        r"""compute_offset(ProfileFitterChi self, Profile model_profile) -> double"""
        return _IMP_saxs.ProfileFitterChi_compute_offset(self, model_profile)

    def resample(self, model_profile, resampled_profile):
        r"""resample(ProfileFitterChi self, Profile model_profile, Profile resampled_profile)"""
        return _IMP_saxs.ProfileFitterChi_resample(self, model_profile, resampled_profile)

    def write_SAXS_fit_file(self, file_name, model_profile, chi_square, c=1, offset=0):
        r"""write_SAXS_fit_file(ProfileFitterChi self, std::string const & file_name, Profile model_profile, double const chi_square, double const c=1, double const offset=0)"""
        return _IMP_saxs.ProfileFitterChi_write_SAXS_fit_file(self, file_name, model_profile, chi_square, c, offset)

    def get_profile(self):
        r"""get_profile(ProfileFitterChi self) -> Profile"""
        return _IMP_saxs.ProfileFitterChi_get_profile(self)
    __swig_destroy__ = _IMP_saxs.delete_ProfileFitterChi

# Register ProfileFitterChi in _IMP_saxs:
_IMP_saxs.ProfileFitterChi_swigregister(ProfileFitterChi)

class WeightedProfileFitterChi(ProfileFitterChi):
    r"""Proxy of C++ IMP::saxs::WeightedProfileFitter< IMP::saxs::ChiScore > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, exp_profile):
        r"""__init__(WeightedProfileFitterChi self, Profile exp_profile) -> WeightedProfileFitterChi"""
        _IMP_saxs.WeightedProfileFitterChi_swiginit(self, _IMP_saxs.new_WeightedProfileFitterChi(exp_profile))

    def compute_score(self, profiles, weights, use_offset=False, NNLS=True):
        r"""compute_score(WeightedProfileFitterChi self, IMP::saxs::ProfilesTemp const & profiles, IMP::Vector< double > & weights, bool use_offset=False, bool NNLS=True) -> double"""
        return _IMP_saxs.WeightedProfileFitterChi_compute_score(self, profiles, weights, use_offset, NNLS)

    def fit_profile(self, partial_profiles, min_c1=0.95, max_c1=1.05, min_c2=-2.0, max_c2=4.0, use_offset=False):
        r"""fit_profile(WeightedProfileFitterChi self, IMP::saxs::ProfilesTemp partial_profiles, double min_c1=0.95, double max_c1=1.05, double min_c2=-2.0, double max_c2=4.0, bool use_offset=False) -> WeightedFitParameters"""
        return _IMP_saxs.WeightedProfileFitterChi_fit_profile(self, partial_profiles, min_c1, max_c1, min_c2, max_c2, use_offset)

    def write_fit_file(self, partial_profiles, fp, fit_file_name, use_offset=False):
        r"""write_fit_file(WeightedProfileFitterChi self, IMP::saxs::ProfilesTemp partial_profiles, WeightedFitParameters fp, std::string const fit_file_name, bool use_offset=False)"""
        return _IMP_saxs.WeightedProfileFitterChi_write_fit_file(self, partial_profiles, fp, fit_file_name, use_offset)
    __swig_destroy__ = _IMP_saxs.delete_WeightedProfileFitterChi

# Register WeightedProfileFitterChi in _IMP_saxs:
_IMP_saxs.WeightedProfileFitterChi_swigregister(WeightedProfileFitterChi)

class Restraint(IMP.Restraint):
    r"""Proxy of C++ IMP::saxs::Restraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(Restraint self, IMP::Particles const & particles, Profile exp_profile, IMP::saxs::FormFactorType ff_type=HEAVY_ATOMS) -> Restraint"""
        _IMP_saxs.Restraint_swiginit(self, _IMP_saxs.new_Restraint(*args))

        if self.__class__ != Restraint:
            _director_objects.register(self)




    def do_get_inputs(self):
        r"""do_get_inputs(Restraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_saxs.Restraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(Restraint self) -> VersionInfo"""
        return _IMP_saxs.Restraint_get_version_info(self)

# Register Restraint in _IMP_saxs:
_IMP_saxs.Restraint_swigregister(Restraint)

class RadiusOfGyrationRestraint(Restraint):
    r"""Proxy of C++ IMP::saxs::RadiusOfGyrationRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, particles, exp_profile, end_q_rg=1.3):
        r"""__init__(RadiusOfGyrationRestraint self, IMP::Particles const & particles, Profile exp_profile, double const end_q_rg=1.3) -> RadiusOfGyrationRestraint"""
        _IMP_saxs.RadiusOfGyrationRestraint_swiginit(self, _IMP_saxs.new_RadiusOfGyrationRestraint(particles, exp_profile, end_q_rg))

# Register RadiusOfGyrationRestraint in _IMP_saxs:
_IMP_saxs.RadiusOfGyrationRestraint_swigregister(RadiusOfGyrationRestraint)

class FloatDistribution(DistBase):
    r"""Proxy of C++ IMP::saxs::Distribution< double > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(FloatDistribution self, double bin_size=pr_resolution) -> FloatDistribution"""
        _IMP_saxs.FloatDistribution_swiginit(self, _IMP_saxs.new_FloatDistribution(*args))

    def get_max_distance(self):
        r"""get_max_distance(FloatDistribution self) -> double"""
        return _IMP_saxs.FloatDistribution_get_max_distance(self)

    def get_bin_size(self):
        r"""get_bin_size(FloatDistribution self) -> double"""
        return _IMP_saxs.FloatDistribution_get_bin_size(self)

    def get_index_from_distance(self, dist):
        r"""get_index_from_distance(FloatDistribution self, double dist) -> unsigned int"""
        return _IMP_saxs.FloatDistribution_get_index_from_distance(self, dist)

    def get_distance_from_index(self, index):
        r"""get_distance_from_index(FloatDistribution self, unsigned int index) -> double"""
        return _IMP_saxs.FloatDistribution_get_distance_from_index(self, index)
    __swig_destroy__ = _IMP_saxs.delete_FloatDistribution

# Register FloatDistribution in _IMP_saxs:
_IMP_saxs.FloatDistribution_swigregister(FloatDistribution)

class VectorDistribution(object):
    r"""Proxy of C++ IMP::saxs::Distribution< IMP::algebra::Vector3D > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(VectorDistribution self, double bin_size=pr_resolution) -> VectorDistribution"""
        _IMP_saxs.VectorDistribution_swiginit(self, _IMP_saxs.new_VectorDistribution(*args))

    def get_max_distance(self):
        r"""get_max_distance(VectorDistribution self) -> double"""
        return _IMP_saxs.VectorDistribution_get_max_distance(self)

    def get_bin_size(self):
        r"""get_bin_size(VectorDistribution self) -> double"""
        return _IMP_saxs.VectorDistribution_get_bin_size(self)

    def get_index_from_distance(self, dist):
        r"""get_index_from_distance(VectorDistribution self, double dist) -> unsigned int"""
        return _IMP_saxs.VectorDistribution_get_index_from_distance(self, dist)

    def get_distance_from_index(self, index):
        r"""get_distance_from_index(VectorDistribution self, unsigned int index) -> double"""
        return _IMP_saxs.VectorDistribution_get_distance_from_index(self, index)
    __swig_destroy__ = _IMP_saxs.delete_VectorDistribution

# Register VectorDistribution in _IMP_saxs:
_IMP_saxs.VectorDistribution_swigregister(VectorDistribution)

class RadialDistributionFunction(FloatDistribution):
    r"""Proxy of C++ IMP::saxs::RadialDistributionFunction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(RadialDistributionFunction self, double bin_size=pr_resolution) -> RadialDistributionFunction
        __init__(RadialDistributionFunction self, std::string const & file_name) -> RadialDistributionFunction
        """
        _IMP_saxs.RadialDistributionFunction_swiginit(self, _IMP_saxs.new_RadialDistributionFunction(*args))

    def scale(self, c):
        r"""scale(RadialDistributionFunction self, double c)"""
        return _IMP_saxs.RadialDistributionFunction_scale(self, c)

    def add(self, model_pr):
        r"""add(RadialDistributionFunction self, RadialDistributionFunction model_pr)"""
        return _IMP_saxs.RadialDistributionFunction_add(self, model_pr)

    def show(self, *args):
        r"""show(RadialDistributionFunction self, _ostream out=std::cout)"""
        return _IMP_saxs.RadialDistributionFunction_show(self, *args)

    def R_factor_score(self, *args):
        r"""R_factor_score(RadialDistributionFunction self, RadialDistributionFunction model_pr, std::string const & file_name="") -> double"""
        return _IMP_saxs.RadialDistributionFunction_R_factor_score(self, *args)

    def fit(self, *args):
        r"""fit(RadialDistributionFunction self, RadialDistributionFunction model_pr, std::string const & file_name="") -> double"""
        return _IMP_saxs.RadialDistributionFunction_fit(self, *args)

    def normalize(self):
        r"""normalize(RadialDistributionFunction self)"""
        return _IMP_saxs.RadialDistributionFunction_normalize(self)

    def add_to_distribution(self, dist, value):
        r"""add_to_distribution(RadialDistributionFunction self, double dist, double value)"""
        return _IMP_saxs.RadialDistributionFunction_add_to_distribution(self, dist, value)
    __swig_destroy__ = _IMP_saxs.delete_RadialDistributionFunction

# Register RadialDistributionFunction in _IMP_saxs:
_IMP_saxs.RadialDistributionFunction_swigregister(RadialDistributionFunction)

class DeltaDistributionFunction(VectorDistribution):
    r"""Proxy of C++ IMP::saxs::DeltaDistributionFunction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(DeltaDistributionFunction self, IMP::Particles const & particles, double max_distance=0.0, double bin_size=pr_resolution) -> DeltaDistributionFunction"""
        _IMP_saxs.DeltaDistributionFunction_swiginit(self, _IMP_saxs.new_DeltaDistributionFunction(*args))

    def calculate_derivative_distribution(self, particle):
        r"""calculate_derivative_distribution(DeltaDistributionFunction self, Particle particle)"""
        return _IMP_saxs.DeltaDistributionFunction_calculate_derivative_distribution(self, particle)

    def show(self, *args):
        r"""show(DeltaDistributionFunction self, _ostream out=std::cout, std::string prefix="")"""
        return _IMP_saxs.DeltaDistributionFunction_show(self, *args)
    __swig_destroy__ = _IMP_saxs.delete_DeltaDistributionFunction

# Register DeltaDistributionFunction in _IMP_saxs:
_IMP_saxs.DeltaDistributionFunction_swigregister(DeltaDistributionFunction)

class SolventAccessibleSurface(object):
    r"""Proxy of C++ IMP::saxs::SolventAccessibleSurface class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def get_solvent_accessibility(self, points, probe_radius=1.8, density=5.0):
        r"""get_solvent_accessibility(SolventAccessibleSurface self, IMP::core::XYZRs const & points, double probe_radius=1.8, double density=5.0) -> IMP::Vector< double >"""
        return _IMP_saxs.SolventAccessibleSurface_get_solvent_accessibility(self, points, probe_radius, density)

    def __init__(self):
        r"""__init__(SolventAccessibleSurface self) -> SolventAccessibleSurface"""
        _IMP_saxs.SolventAccessibleSurface_swiginit(self, _IMP_saxs.new_SolventAccessibleSurface())
    __swig_destroy__ = _IMP_saxs.delete_SolventAccessibleSurface

# Register SolventAccessibleSurface in _IMP_saxs:
_IMP_saxs.SolventAccessibleSurface_swigregister(SolventAccessibleSurface)

class ProfileFitterChiLog(IMP.Object):
    r"""Proxy of C++ IMP::saxs::ProfileFitter< IMP::saxs::ChiScoreLog > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(ProfileFitterChiLog self, Profile exp_profile) -> ProfileFitterChiLog
        __init__(ProfileFitterChiLog self, Profile exp_profile, ChiScoreLog sf) -> ProfileFitterChiLog
        """
        _IMP_saxs.ProfileFitterChiLog_swiginit(self, _IMP_saxs.new_ProfileFitterChiLog(*args))

    def compute_score(self, *args):
        r"""compute_score(ProfileFitterChiLog self, Profile model_profile, bool use_offset=False, std::string const fit_file_name="") -> double"""
        return _IMP_saxs.ProfileFitterChiLog_compute_score(self, *args)

    def fit_profile(self, *args):
        r"""fit_profile(ProfileFitterChiLog self, Profile partial_profile, double min_c1=0.95, double max_c1=1.05, double min_c2=-2.0, double max_c2=4.0, bool use_offset=False, std::string const fit_file_name="") -> FitParameters"""
        return _IMP_saxs.ProfileFitterChiLog_fit_profile(self, *args)

    def compute_scale_factor(self, model_profile, offset=0.0):
        r"""compute_scale_factor(ProfileFitterChiLog self, Profile model_profile, double offset=0.0) -> double"""
        return _IMP_saxs.ProfileFitterChiLog_compute_scale_factor(self, model_profile, offset)

    def compute_offset(self, model_profile):
        r"""compute_offset(ProfileFitterChiLog self, Profile model_profile) -> double"""
        return _IMP_saxs.ProfileFitterChiLog_compute_offset(self, model_profile)

    def resample(self, model_profile, resampled_profile):
        r"""resample(ProfileFitterChiLog self, Profile model_profile, Profile resampled_profile)"""
        return _IMP_saxs.ProfileFitterChiLog_resample(self, model_profile, resampled_profile)

    def write_SAXS_fit_file(self, file_name, model_profile, chi_square, c=1, offset=0):
        r"""write_SAXS_fit_file(ProfileFitterChiLog self, std::string const & file_name, Profile model_profile, double const chi_square, double const c=1, double const offset=0)"""
        return _IMP_saxs.ProfileFitterChiLog_write_SAXS_fit_file(self, file_name, model_profile, chi_square, c, offset)

    def get_profile(self):
        r"""get_profile(ProfileFitterChiLog self) -> Profile"""
        return _IMP_saxs.ProfileFitterChiLog_get_profile(self)
    __swig_destroy__ = _IMP_saxs.delete_ProfileFitterChiLog

# Register ProfileFitterChiLog in _IMP_saxs:
_IMP_saxs.ProfileFitterChiLog_swigregister(ProfileFitterChiLog)

class ProfileFitterRatioVolatility(IMP.Object):
    r"""Proxy of C++ IMP::saxs::ProfileFitter< IMP::saxs::RatioVolatilityScore > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(ProfileFitterRatioVolatility self, Profile exp_profile) -> ProfileFitterRatioVolatility
        __init__(ProfileFitterRatioVolatility self, Profile exp_profile, IMP::saxs::RatioVolatilityScore * sf) -> ProfileFitterRatioVolatility
        """
        _IMP_saxs.ProfileFitterRatioVolatility_swiginit(self, _IMP_saxs.new_ProfileFitterRatioVolatility(*args))

    def compute_score(self, *args):
        r"""compute_score(ProfileFitterRatioVolatility self, Profile model_profile, bool use_offset=False, std::string const fit_file_name="") -> double"""
        return _IMP_saxs.ProfileFitterRatioVolatility_compute_score(self, *args)

    def fit_profile(self, *args):
        r"""fit_profile(ProfileFitterRatioVolatility self, Profile partial_profile, double min_c1=0.95, double max_c1=1.05, double min_c2=-2.0, double max_c2=4.0, bool use_offset=False, std::string const fit_file_name="") -> FitParameters"""
        return _IMP_saxs.ProfileFitterRatioVolatility_fit_profile(self, *args)

    def compute_scale_factor(self, model_profile, offset=0.0):
        r"""compute_scale_factor(ProfileFitterRatioVolatility self, Profile model_profile, double offset=0.0) -> double"""
        return _IMP_saxs.ProfileFitterRatioVolatility_compute_scale_factor(self, model_profile, offset)

    def compute_offset(self, model_profile):
        r"""compute_offset(ProfileFitterRatioVolatility self, Profile model_profile) -> double"""
        return _IMP_saxs.ProfileFitterRatioVolatility_compute_offset(self, model_profile)

    def resample(self, model_profile, resampled_profile):
        r"""resample(ProfileFitterRatioVolatility self, Profile model_profile, Profile resampled_profile)"""
        return _IMP_saxs.ProfileFitterRatioVolatility_resample(self, model_profile, resampled_profile)

    def write_SAXS_fit_file(self, file_name, model_profile, chi_square, c=1, offset=0):
        r"""write_SAXS_fit_file(ProfileFitterRatioVolatility self, std::string const & file_name, Profile model_profile, double const chi_square, double const c=1, double const offset=0)"""
        return _IMP_saxs.ProfileFitterRatioVolatility_write_SAXS_fit_file(self, file_name, model_profile, chi_square, c, offset)

    def get_profile(self):
        r"""get_profile(ProfileFitterRatioVolatility self) -> Profile"""
        return _IMP_saxs.ProfileFitterRatioVolatility_get_profile(self)
    __swig_destroy__ = _IMP_saxs.delete_ProfileFitterRatioVolatility

# Register ProfileFitterRatioVolatility in _IMP_saxs:
_IMP_saxs.ProfileFitterRatioVolatility_swigregister(ProfileFitterRatioVolatility)


def get_module_version():
    r"""get_module_version() -> std::string const"""
    return _IMP_saxs.get_module_version()

def get_example_path(fname):
    r"""get_example_path(std::string fname) -> std::string"""
    return _IMP_saxs.get_example_path(fname)

def get_data_path(fname):
    r"""get_data_path(std::string fname) -> std::string"""
    return _IMP_saxs.get_data_path(fname)

from . import _version_check
_version_check.check_version(get_module_version())
__version__ = get_module_version()






## \example algebra/grid_space.py
# This example shows how to use the grid support in IMP.algebra to
# discretize a set of continuous points. In this case the points are
# simply randomly drawn from the surface of a sphere, but they could be
# taken from something more interesting.

from __future__ import print_function
import IMP.algebra
import sys

IMP.setup_from_argv(sys.argv, "grid space")

# create a unit grid with its origin at 0,0,0
g = IMP.algebra.SparseUnboundedIntGrid3D(1, IMP.algebra.Vector3D(0, 0, 0))

s = IMP.algebra.Sphere3D(IMP.algebra.Vector3D(1, 1, 1), 6)
count = 0
for i in range(0, 100):
    p = IMP.algebra.get_random_vector_on(s)
    ei = g.get_extended_index(p)
    if g.get_has_index(ei):
        print("hit")
    else:
        g.add_voxel(ei, count)
        count += 1

in_count = 0
for i in g.get_extended_indexes(IMP.algebra.get_bounding_box(s)):
    if IMP.algebra.get_distance(s.get_center(), g.get_center(i)) > 6:
        continue
    if g.get_has_index(i):
        print("hit")
    else:
        g.add_voxel(i, -1)
        in_count += 1

print("There are", len(g.get_all_indexes()), "distinct values", count, in_count)



## \example core/rigid_collisions.py
# Collision detection and building of a non-bonded list can be sped up when rigid bodies are used.
# To do this, use a RigidClosePairsFinder.

import IMP
import IMP.core
import IMP.atom
import IMP.container
import sys

IMP.setup_from_argv(sys.argv, "rigid collisions")

# This example addes a restraint on nonbonded interactions
# Since it is between two rigid bodies, internal interactions are ignored

m = IMP.Model()
# The particles in the rigid bodies
rbps0 = IMP.core.create_xyzr_particles(m, 3, 1)
rbps1 = IMP.core.create_xyzr_particles(m, 3, 1)

rbp0 = IMP.Particle(m)
rbp1 = IMP.Particle(m)

rbss0 = IMP.core.RigidBody.setup_particle(rbp0, IMP.core.XYZs(rbps0))

rbss1 = IMP.core.RigidBody.setup_particle(rbp1, IMP.core.XYZs(rbps1))

lsc = IMP.container.ListSingletonContainer(m)
lsc.add(rbps0)
lsc.add(rbps1)

# Set up the nonbonded list
nbl = IMP.container.ClosePairContainer(
    lsc, 0, IMP.core.RigidClosePairsFinder(), 2.0)

# Set up excluded volume
ps = IMP.core.SphereDistancePairScore(IMP.core.HarmonicLowerBound(0, 1))
evr = IMP.container.PairsRestraint(ps, nbl)
sf = IMP.core.RestraintsScoringFunction([evr])

# Set up optimizer
o = IMP.core.ConjugateGradients(m)
o.set_scoring_function(sf)

done = False
while not done:
    try:
        o.optimize(1000)
    except IMP.ModelException:
        for d in [rbss0, rbss1]:
            d.set_transformation(
                IMP.algebra.Transformation3D(
                    IMP.algebra.get_random_rotation_3d(),
                    IMP.algebra.get_random_vector_in(
                        IMP.algebra.BoundingBox3D(
                            IMP.algebra.Vector3D(
                                0, 0, 0),
                            IMP.algebra.Vector3D(10, 10, 10)))))
    else:
        done = True



## \example atom/assess_dope.py
# The script shows how to assess a protein conformation using DOPE.

from __future__ import print_function
import IMP
import IMP.atom
import IMP.container
import sys

IMP.setup_from_argv(sys.argv, "assess dope")

def create_representation():
    m = IMP.Model()
    mp0 = IMP.atom.read_pdb(IMP.atom.get_example_path(
        '1fdx.B99990001.pdb'), m, IMP.atom.NonWaterNonHydrogenPDBSelector())
    prot = IMP.atom.get_by_type(mp0, IMP.atom.CHAIN_TYPE)[0]
    return (m, prot)


def add_dope(m, prot):
    ps = IMP.atom.get_by_type(prot, IMP.atom.ATOM_TYPE)
    for p in ps:
        if not IMP.atom.Atom.get_is_setup(p):
            print("Huh?", p)
    dpc = IMP.container.ClosePairContainer(ps, 15.0, 1.0)
    # By default the score is evaluated on all nearby pairs of atoms,
    # even atoms that are bonded (or related by angles or dihedrals).
    # This is consistent with the behavior of the score in MODELLER.
    # If you like you can exclude pairs of atoms belonging to the same residue
    # by adding a pair filter to the container:
    # f = IMP.atom.SameResiduePairFilter()
    # dpc.add_pair_filter(f)
    IMP.atom.add_dope_score_data(prot)
    dps = IMP.atom.DopePairScore(15.0)
    d = IMP.container.PairsRestraint(dps, dpc)
    return d

print("creating representation")
(m, prot) = create_representation()

print("creating DOPE score function")
d = add_dope(m, prot)

IMP.set_check_level(IMP.USAGE)
print("DOPE SCORE ::", d.evaluate(False))



## \example kernel/graph.py
# A simple example showing how to use the graph interface for in python.

from __future__ import print_function
import IMP
import sys

IMP.setup_from_argv(sys.argv, "Graph interface")

m = IMP.Model()
# An undirected graph with an IMP::Object for each node
g = IMP.DependencyGraph()
vs = []
ps = []
for i in range(0, 10):
    ps.append(IMP.Particle(m))
    vs.append(g.add_vertex(ps[-1]))
g.add_edge(vs[0], vs[1])
g.add_edge(vs[1], vs[2])

# try to use the altgraph package to visualize
IMP.show_graphviz(g)
try:
    IMP.show_graphviz(g)
except:
    print("Oh well, no altgraph")

try:
    import matplotlib
    # the engine to be used must be selected before pyplot is imported
    matplotlib.use("macosx")
    import matplotlib.pyplot

    # the method below requires the altgraph python package
    xg = IMP.get_networkx_graph(g)

    # the networkx visualization tools suck, so skip them
    #import networkx
    # networkx.draw(xg)
    # networkx.draw_shell(xg)
    # matplotlib.pyplot.show()
except:
    print("networkx not fully installed")

g.remove_vertex(0)

# we can also try another show method
try:
    IMP.show_graphviz(g)
except:
    print("oh well, something not working with graphviz")

# finally, we can
# in and out neighbors are the same
for n in g.get_in_neighbors(8):
    print(g.get_vertex_name(n).get_name())



## \example core/XYZ_Decorator.py
# This is a simple example using the XYZ decorator to set the coordinates
# of some particles and compute the distance between them.

import IMP
import IMP.core
import IMP.algebra
import sys

IMP.setup_from_argv(sys.argv, "XYZ decorator example")

m = IMP.Model()
p0 = m.add_particle("xyz0")
# add x,y,z coordinates to the particle
d0 = IMP.core.XYZ.setup_particle(m, p0, IMP.algebra.Vector3D(0, 1, 2))
p1 = m.add_particle("xyz1")
# add slots for the coordinates, but leave them uninitialized
d1 = IMP.core.XYZ.setup_particle(m, p1)
# set the coordinate values
d1.set_coordinates(IMP.algebra.Vector3D(3, 4, 5))

# return the distance between the two points described by the decorated
# particles
print(IMP.core.get_distance(d0, d1))



"""@namespace IMP.EMageFit.csv_related
   Utility functions to handle CSV files.
"""

import csv


def is_comment(list_values, comment_char="#"):
    if(len(list_values) == 0 or list_values[0][0] == comment_char):
        return True
    return False


def read_csv(fn, delimiter="|", comment_char="#", max_number=False):
    """
        Simple reader of csv files that disregards lines with comments
    """
    f = open(fn, "r")
    reader = csv.reader(f, delimiter=delimiter)
    if(max_number):
        i = 1
        rows = []
        for r in reader:
            if(i > max_number):
                break
            rows.append(r)
            i += 1
    else:
        rows = [d for d in reader if(not is_comment(d, comment_char))]
    f.close()
    return rows


def read_csv_keyword(fn_or_f, keyword, delimiter="|",
                     comment_char="#"):
    """
        Reader of csv files that only recovers lines starting with a keyword

    """
    if(isinstance(fn_or_f, str)):
        f = open(fn_or_f, "r")
    elif(isinstance(fn_or_f, file)):
        f = fn_or_f

    reader = csv.reader(f, delimiter=delimiter)
    rows = []
    for row in reader:
        if(not is_comment(row) and row[0] == keyword):
            rows.append(row)

    if(isinstance(fn_or_f, str)):
        f.close()

    return rows
